package uk.ac.ox.cs.pdq.planner.dag.cost;

import uk.ac.ox.cs.pdq.cost.estimators.WhiteBoxCostEstimator;
import uk.ac.ox.cs.pdq.cost.statistics.estimators.CardinalityEstimator;
import uk.ac.ox.cs.pdq.logging.performance.StatisticsCollector;
import uk.ac.ox.cs.pdq.plan.Cost;
import uk.ac.ox.cs.pdq.plan.DAGPlan;
import uk.ac.ox.cs.pdq.reasoning.Configuration;

import com.google.common.eventbus.EventBus;


/**
 * A whitebox cost estimation implementation.
 * According to this, the cost of a plan equals the number of the accesses.
 * Based on cost estimation as defined in the "Database Management System" by
 * Ramakrishnan and Gehrke.
 *
 * Roughly speaking, the cost of a plan is the sum of the cost of its sub-plans,
 * plus it own IO cost multiple by an estimation of its cardinality.
 *
 * @author Julien Leblay
 *
 */
public final class ConfigurationWhiteBoxCostEstimator extends WhiteBoxCostEstimator<DAGPlan> implements ConfigurationCostEstimator{

	/**
	 * Default constructor
	 * @param eventBus
	 * @param collectStats
	 * @param ce CardinalityEstimator
	 */
	public ConfigurationWhiteBoxCostEstimator(EventBus eventBus, boolean collectStats, CardinalityEstimator ce) {
		super(eventBus, collectStats, ce);
	}

	/**
	 * Constructor
	 * @param stats
	 * @param ce CardinalityEstimator
	 */
	private ConfigurationWhiteBoxCostEstimator(StatisticsCollector stats, CardinalityEstimator ce) {
		super(stats, ce);
	}

	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	@Override
	public ConfigurationWhiteBoxCostEstimator clone() {
		return new ConfigurationWhiteBoxCostEstimator(this.stats.clone(), this.cardEstimator.clone());
	}

	@Override
	public Cost cost(Configuration configuration) {
		return super.cost((DAGPlan) configuration.getPlan());
	}

	@Override
	public Cost estimateCost(Configuration configuration) {
		return super.estimateCost((DAGPlan) configuration.getPlan());
	}

}
