package uk.ac.ox.cs.pdq.planner.dag.equivalence;

import java.util.Collection;
import java.util.Iterator;
import java.util.Map.Entry;

import uk.ac.ox.cs.pdq.planner.dag.DAGChaseConfiguration;
import uk.ac.ox.cs.pdq.planner.reasoning.chase.state.ChaseState;


/**
 * A collection of equivalence classes
 *
 * @author Efthymia Tsamoura
 *
 * @param <S>
 */
public interface DAGEquivalenceClasses<S extends ChaseState> {

	/**
	 * Adds the configuration to its structurally equivalent class
	 * @param configuration
	 */
	void addEntry(DAGChaseConfiguration<S> configuration);

	/**
	 * Removes the input configuration from its class
	 * @param configuration
	 */
	void removeEntry(DAGChaseConfiguration<S> configuration);

	/**
	 * Retains the configurations
	 * @param configurations
	 */
	void retainAll(Collection<DAGChaseConfiguration<S>> configurations);

	/**
	 * @return the configurations of all classes
	 */
	Collection<DAGChaseConfiguration<S>> getConfigurations();

	/**
	 *
	 * @return all classes
	 */
	Collection<DAGEquivalenceClass<S>> getEquivalenceClasses();

	/**
	 * @param configuration
	 * @return the equivalence class of the input configuration
	 */
	DAGEquivalenceClass<S> getEquivalenceClass(DAGChaseConfiguration<S> configuration);

	/**
	 * Removes all input configurations
	 * @param configurations
	 */
	void removeAll(Collection<DAGChaseConfiguration<S>> configurations);

	/**
	 * @param configuration
	 * @return the configurations that are dominated by the input configuration
	 */
	Collection<DAGChaseConfiguration<S>> dominatedBy(DAGChaseConfiguration<S> configuration);

	/**
	 * @param configuration
	 * @return a configuration that is equivalent to the input configuration
	 */
	DAGChaseConfiguration<S> structurallyEquivalentTo(DAGChaseConfiguration<S> configuration);

	/**
	 * @param configuration
	 * @return a configuration that dominates the input configuration
	 */
	DAGChaseConfiguration<S> dominate(DAGChaseConfiguration<S> configuration);

	/**
	 * @param configuration
	 * @return true if at least one class contains the input configuration
	 */
	boolean contains(DAGChaseConfiguration<S> configuration);

	/**
	 * @return an iterator over the classes
	 */
	Iterator<Entry<DAGChaseConfiguration<S>, SynchronizedEquivalenceClass<S>>> iterator();

	/**
	 * @return true if all classes are empty
	 */
	boolean isEmpty();

	void clear();

	/**
	 * @return int
	 */
	int size();

	/**
	 * @return double
	 */
	double averageClassSize();

	/**
	 * @return int
	 */
	int medianClassSize();
}