package uk.ac.ox.cs.pdq.planner.dag.explorer;

import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.Set;

import uk.ac.ox.cs.pdq.plan.Plan;
import uk.ac.ox.cs.pdq.planner.dag.BinaryConfiguration;
import uk.ac.ox.cs.pdq.planner.dag.ConfigurationUtility;
import uk.ac.ox.cs.pdq.planner.dag.DAGChaseConfiguration;
import uk.ac.ox.cs.pdq.planner.dag.DAGConfiguration;
import uk.ac.ox.cs.pdq.planner.reasoning.chase.state.ChaseState;

/**
 * Utility class
 * @author Efthymia Tsamoura
 *
 */
public class ExplorerUtils {

	/**
	 *
	 * @param configurations
	 * @param input
	 * @return
	 * 		the configuration that dominates the input
	 */
	public static <S extends ChaseState> DAGChaseConfiguration<S> isDominated(Collection<DAGChaseConfiguration<S>> configurations, DAGChaseConfiguration<?> input) {
		for(DAGChaseConfiguration<S> configuration:configurations) {
			if(input.isDominatedBy(configuration)) {
				return configuration;
			}
		}
		return null;
	}

	/**
	 * @param input 
	 * @return the binary subconfigurations of the input configuration
	 */
	public static <S extends ChaseState> Set<BinaryConfiguration<S>> getBinarySubconfigurations(Set<DAGChaseConfiguration<S>> input) {
		Set<BinaryConfiguration<S>> subconfigurations = new LinkedHashSet<>();
		for(DAGChaseConfiguration<S> configuration:input) {
			for(DAGConfiguration subconfiguration:configuration.getSubconfigurations()) {
				if(subconfiguration instanceof BinaryConfiguration) {
					subconfigurations.add((BinaryConfiguration<S>) subconfiguration);
				}
			}
		}
		return subconfigurations;
	}

	/**
	 * @param left DAGChaseConfiguration<S>
	 * @param right DAGChaseConfiguration<S>
	 * @param bestPlan P
	 * @return true if the configuration composed by the input configurations has the potential to lead to the minimum cost plan
	 */
	public static <P extends Plan, S extends ChaseState> Boolean getPotential(DAGChaseConfiguration<S> left, DAGChaseConfiguration<S> right, P bestPlan) {
		return ConfigurationUtility.getPotential(left, right, bestPlan, left.getCostEstimator(), left.getSuccessDominanceDetector());
	}

	/**
	 * @param configuration DAGChaseConfiguration<S>
	 * @param bestPlan P
	 * @return true if the input configuration has the potential to lead to the minimum cost plan
	 */
	public static <P extends Plan, S extends ChaseState> Boolean getPotential(DAGChaseConfiguration<S> configuration, P bestPlan) {
		return bestPlan == null
				|| !configuration.getSuccessDominanceDetector().isDominated(configuration.getPlan(), bestPlan);
	}

}
