package uk.ac.ox.cs.pdq.planner.dag.explorer;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.commons.lang3.tuple.Pair;

import uk.ac.ox.cs.pdq.planner.dag.DAGChaseConfiguration;
import uk.ac.ox.cs.pdq.planner.dag.equivalence.DAGEquivalenceClass;
import uk.ac.ox.cs.pdq.planner.reasoning.chase.state.ChaseState;

/**
 * Map of representatives. For each configuration c = BinConfiguration(c_1,c_2) we create a map from the
 * equivalence classes of c and c' to c''. This map helps us reducing the chasing time, i.e.,
 * if c'_1 and c'_2 are structurally equivalent to c_1 and c_2, respectively, and
 * c = BinConfiguration(c_1,c_2) has already been fully chased,
 * then we copy the state of c to the state of c' = BinConfiguration(c'_1,c'_2).
 * 
 * @author Efthymia Tsamoura
 *
 * @param <S>
 */
public class Representative<S extends ChaseState> {
	
	private final Map<Pair<DAGEquivalenceClass<S>,DAGEquivalenceClass<S>>,DAGChaseConfiguration<S>> representatives  = new ConcurrentHashMap<>();

	public DAGChaseConfiguration<S> getRepresentative(DAGChaseConfiguration<S> left, DAGChaseConfiguration<S> right) {
		//The equivalence class of the left input configuration
		DAGEquivalenceClass<S> rep0 = left.getEquivalenceClass();
		//The equivalence class of the left input configuration
		DAGEquivalenceClass<S> rep1 = right.getEquivalenceClass();
		//A configuration BinConfiguration(c,c'), where c and c' belong to the equivalence classes of
		//the left and right input configuration, respectively.
		return this.representatives.get(Pair.of(rep0, rep1));
	}
	
	public void put(DAGChaseConfiguration<S> left, DAGChaseConfiguration<S> right, DAGChaseConfiguration<S> representative) {
		this.representatives.put(Pair.of(left.getEquivalenceClass(), right.getEquivalenceClass()), representative);
	}
}
