package uk.ac.ox.cs.pdq.planner.dag.explorer.filters;

import java.util.Collection;
import java.util.LinkedHashSet;

import uk.ac.ox.cs.pdq.planner.dag.DAGChaseConfiguration;
import uk.ac.ox.cs.pdq.planner.reasoning.chase.state.ChaseState;

/**
 * Filters out the configurations with depth > the depth threshold
 * @author Efthymia Tsamoura
 *
 * @param <S>
 */
public class DepthFilter<S extends ChaseState> implements Filter<S> {

	/** Depth threshold. Configurations with depth > this threshold will be filtered out*/
	private final int depthThreshold;

	public DepthFilter() {
		this.depthThreshold = 3;
	}

	/**
	 * Constructor for DepthFilter.
	 * @param depthThreshold int
	 */
	public DepthFilter(int depthThreshold) {
		this.depthThreshold = depthThreshold;
	}

	/**
	 * @param configurations Collection<DAGConfiguration<S>>
	 * @return Collection<DAGConfiguration<S>>
	 * @see uk.ac.ox.cs.pdq.dag.explorer.filters.Filter#filter(Collection<DAGConfiguration<S>>)
	 */
	@Override
	public Collection<DAGChaseConfiguration<S>> filter(Collection<DAGChaseConfiguration<S>> configurations) {
		Collection<DAGChaseConfiguration<S>> filtered = new LinkedHashSet<>();
		for(DAGChaseConfiguration<S> configuration: configurations) {
			if(configuration.getHigh() >= this.depthThreshold) {
				filtered.add(configuration);
			}
		}
		return filtered;
	}


}
