package uk.ac.ox.cs.pdq.planner.dag.explorer.filters;

import uk.ac.ox.cs.pdq.planner.PlannerParameters.FilterTypes;
import uk.ac.ox.cs.pdq.planner.reasoning.chase.state.ChaseState;

/**
 * Creates filters based on the input arguments.
 * The available options are:
 * 		FACT_THROTTLING for a fact throttling filter and
 * 		NUMERICAL_THROTTLING for a numerical fact throttling filter
 * @author Efthymia Tsamoura
 *
 */
public class FilterFactory<S extends ChaseState> {

	private final FilterTypes type;


	/**
	 * Constructor for FilterFactory.
	 * @param type FilterTypes
	 */
	public FilterFactory(FilterTypes type) {
		this.type = type;
	}

	/**
	 * @return Filter<S>
	 */
	public Filter<S> getInstance() {
		if(this.type == null) {
			return null;
		}
		switch(this.type) {
		case FACT_DOMINATED_FILTER:
			return new FactDominationFilter<>();
		case NUMERICALLY_DOMINATED_FILTER:
			return new NumericalFactDominationFilter<>();
		default:
			throw new java.lang.IllegalArgumentException();
		}
	}
}
