package uk.ac.ox.cs.pdq.planner.dag.explorer.filters;

import java.util.Collection;
import java.util.LinkedHashSet;

import org.apache.commons.collections4.CollectionUtils;

import uk.ac.ox.cs.pdq.fol.Constant;
import uk.ac.ox.cs.pdq.fol.Query;
import uk.ac.ox.cs.pdq.planner.dag.DAGChaseConfiguration;
import uk.ac.ox.cs.pdq.planner.reasoning.chase.state.ChaseState;
import uk.ac.ox.cs.pdq.util.Utility;


/**
 * Filters out the non-reachable configurations
 * @author Efthymia Tsamoura
 *
 * @param <S>
 */
public class ReachabilityFilter<S extends ChaseState> implements Filter<S> {

	private final Query<?> query;

	/**
	 * Constructor for ReachabilityFilter.
	 * @param query Query<?>
	 */
	public ReachabilityFilter(Query<?> query) {
		this.query = query;
	}

	/**
	 * @param configurations Collection<DAGConfiguration<S>>
	 * @return Collection<DAGConfiguration<S>>
	 * @see uk.ac.ox.cs.pdq.dag.explorer.filters.Filter#filter(Collection<DAGConfiguration<S>>)
	 */
	@Override
	public Collection<DAGChaseConfiguration<S>> filter(Collection<DAGChaseConfiguration<S>> configurations) {
		Collection<DAGChaseConfiguration<S>> filtered = new LinkedHashSet<>();
		Collection<Constant> canonical = Utility.getConstants(this.query.getCanonicalQuery().getPredicates());
		for(DAGChaseConfiguration<S> configuration: configurations) {
			if(!CollectionUtils.containsAny(configuration.getOutput(), canonical)) {
				filtered.add(configuration);
			}
		}
		return filtered;
	}


}
