package uk.ac.ox.cs.pdq.planner.dag.explorer.validators;

import java.util.ArrayList;
import java.util.List;

import uk.ac.ox.cs.pdq.planner.dag.ConfigurationUtility;
import uk.ac.ox.cs.pdq.planner.dag.DAGChaseConfiguration;
import uk.ac.ox.cs.pdq.planner.reasoning.chase.state.ChaseState;

/**
 * Requires the input pair of configurations to be non trivial
 * @author Efthymia Tsamoura
 *
 * @param <S>
 */
public class DefaultValidator<S extends ChaseState> implements Validator<S>{

	public DefaultValidator() {
	}

	/**
	 * @param left DAGConfiguration<S>
	 * @param right DAGConfiguration<S>
	 * @return boolean
	 * @see uk.ac.ox.cs.pdq.dag.explorer.validators.Validator#validate(DAGConfiguration<S>, DAGConfiguration<S>)
	 */
	@Override
	public boolean validate(DAGChaseConfiguration<S> left, DAGChaseConfiguration<S> right) {
		return ConfigurationUtility.isNonTrivial(left, right);
	}

	/**
	 * @param left DAGConfiguration<S>
	 * @param right DAGConfiguration<S>
	 * @param depth int
	 * @return boolean
	 * @see uk.ac.ox.cs.pdq.dag.explorer.validators.Validator#validate(DAGConfiguration<S>, DAGConfiguration<S>, int)
	 */
	@Override
	public boolean validate(DAGChaseConfiguration<S> left, DAGChaseConfiguration<S> right, int depth) {
		return left.getHigh() + right.getHigh() == depth && this.validate(left, right);
	}

	/**
	 * @return Validator<S>
	 * @see uk.ac.ox.cs.pdq.planner.dag.explorer.validators.Validator#clone()
	 */
	@Override
	public Validator<S> clone() {
		return new DefaultValidator<>();
	}

	/**
	 * @param input
	 * @return a deep copy of the input validators list
	 */
	public static <S extends ChaseState> List<Validator<S>> deepCopy(List<Validator<S>> input) {
		List<Validator<S>> list = new ArrayList<>();
		for(int i = 0; i < input.size(); ++i) {
			list.add(input.get(i).clone());
		}
		return list;
	}
}
