package uk.ac.ox.cs.pdq.planner.dag.explorer.validators;

import uk.ac.ox.cs.pdq.planner.dag.ApplyRule;
import uk.ac.ox.cs.pdq.planner.dag.ConfigurationUtility;
import uk.ac.ox.cs.pdq.planner.dag.DAGChaseConfiguration;
import uk.ac.ox.cs.pdq.planner.reasoning.chase.state.ChaseState;


/**
 * Requires the input pair of configurations to be non trivial
 * and their composition to be a closed left-deep configuration
 * @author Efthymia Tsamoura
 *
 * @param <S>
 */
public class LinearValidator<S extends ChaseState> implements Validator<S>{

	public LinearValidator() {
	}

	/**
	 * @param left DAGConfiguration<S>
	 * @param right DAGConfiguration<S>
	 * @return boolean
	 * @see uk.ac.ox.cs.pdq.dag.explorer.validators.Validator#validate(DAGConfiguration<S>, DAGConfiguration<S>)
	 */
	@Override
	public boolean validate(DAGChaseConfiguration<S> left, DAGChaseConfiguration<S> right) {
		return  right instanceof ApplyRule
				&& left.isClosed()
				&& left.getOutput().containsAll(right.getInput())
				&& ConfigurationUtility.isNonTrivial(left, right);
	}

	/**
	 * @param left DAGConfiguration<S>
	 * @param right DAGConfiguration<S>
	 * @param depth int
	 * @return boolean
	 * @see uk.ac.ox.cs.pdq.dag.explorer.validators.Validator#validate(DAGConfiguration<S>, DAGConfiguration<S>, int)
	 */
	@Override
	public boolean validate(DAGChaseConfiguration<S> left, DAGChaseConfiguration<S> right, int depth) {
		return left.getHigh() + right.getHigh() == depth && this.validate(left, right);
	}

	/**
	 * @return Validator<S>
	 * @see uk.ac.ox.cs.pdq.planner.dag.explorer.validators.Validator#clone()
	 */
	@Override
	public Validator<S> clone() {
		return new LinearValidator<>();
	}

}
