package uk.ac.ox.cs.pdq.planner.dag.potential;

import uk.ac.ox.cs.pdq.plan.Plan;
import uk.ac.ox.cs.pdq.planner.dag.DAGChaseConfiguration;
import uk.ac.ox.cs.pdq.planner.reasoning.chase.dominance.SuccessDominance;
import uk.ac.ox.cs.pdq.planner.reasoning.chase.state.ChaseState;

/**
 * Assesses the potential of a configuration to lead to the minimum-cost configuration
 *
 * @author Efthymia Tsamoura
 *
 * @param <S>
 */
public interface PotentialAssessor<S extends ChaseState> {

	/**
	 * @param configuration
	 * @return true if the input configuration could lead to the minimum-cost configuration
	 */
	Boolean getPotential(DAGChaseConfiguration<S> configuration);

	/**
	 * @param left
	 * @param right
	 * @return true if the configuration composed from the left and right input configurations could lead to the minimum-cost configuration
	 */
	Boolean getPotential(DAGChaseConfiguration<S> left, DAGChaseConfiguration<S> right);

	/**
	 * Updates the best plan. It is used to perform success domination checks
	 * @param bestPlan
	 */
	void update(Plan bestPlan);

	/**
	 * @param successDominance
	 */
	void update(SuccessDominance successDominance);

	/**
	 * @return PotentialAssessor<S>
	 */
	PotentialAssessor<S> clone();
}
