package uk.ac.ox.cs.pdq.planner.dag.priority;

import java.util.Collection;
import java.util.List;
import java.util.Set;

import uk.ac.ox.cs.pdq.planner.dag.DAGChaseConfiguration;
import uk.ac.ox.cs.pdq.planner.dag.explorer.validators.DefaultValidator;
import uk.ac.ox.cs.pdq.planner.dag.explorer.validators.Validator;
import uk.ac.ox.cs.pdq.planner.dag.potential.PotentialAssessor;
import uk.ac.ox.cs.pdq.planner.reasoning.chase.state.ChaseState;

import com.google.common.base.Preconditions;
import com.google.common.collect.Sets;

/**
 * Returns all configurations that satisfy given shape restrictions
 *
 * @author Efthymia Tsamoura
 *
 * @param <S>
 */
public class DefaultPriorityAssessor<S extends ChaseState> extends PriorityAssessor<S>{

	/**
	 *
	 * @param validators
	 * 		Validate pairs of configurations to be composed
	 * @param potential
	 * 		Assesses the potential of a configuration to lead to the minimum-cost configuration
	 */
	public DefaultPriorityAssessor(
			List<Validator<S>> validators,
			PotentialAssessor<S> potential) {
		super(validators, potential, Integer.MAX_VALUE);
	}

	/**
	 *
	 * @param left
	 * @param right
	 * 		Candidate configurations to combine with the left input one
	 * @param depth
	 * 		The target depth of the composition
	 * @return all configurations in the right input collection that satisfy given shape restrictions
	 */
	@Override
	public Collection<DAGChaseConfiguration<S>> select(DAGChaseConfiguration<S> left, Collection<DAGChaseConfiguration<S>> right, int depth) {
		Set<DAGChaseConfiguration<S>> selected = Sets.newLinkedHashSet();
		for(DAGChaseConfiguration<S> configuration:right) {
			Preconditions.checkNotNull(configuration.getEquivalenceClass());
			Preconditions.checkState(!configuration.getEquivalenceClass().isEmpty());
			if(!configuration.getEquivalenceClass().isSleeping() &&
					this.validate(left, configuration, depth) &&
					this.getPotential().getPotential(left, configuration) ){
				selected.add(configuration);
			}
		}
		return selected;
	}

	/**
	 * @return DefaultPriorityAssessor<S>
	 */
	@Override
	public DefaultPriorityAssessor<S> clone() {
		return new DefaultPriorityAssessor<>(DefaultValidator.deepCopy(this.getValidators()), this.getPotential().clone());
	}
}