package uk.ac.ox.cs.pdq.planner.db.access;

import java.util.ArrayList;
import java.util.List;

import uk.ac.ox.cs.pdq.db.Attribute;
import uk.ac.ox.cs.pdq.db.LinearGuarded;
import uk.ac.ox.cs.pdq.db.Relation;
import uk.ac.ox.cs.pdq.fol.Conjunction;
import uk.ac.ox.cs.pdq.fol.PredicateFormula;
import uk.ac.ox.cs.pdq.fol.Variable;
import uk.ac.ox.cs.pdq.planner.db.access.AccessibleSchema.AccessedRelation;
import uk.ac.ox.cs.pdq.planner.db.access.AccessibleSchema.AccessibleRelation;

/**
 * A defining axiom
 *
 * @author Efthymia Tsamoura
 * @author Julien Leblay
 */
public class DefiningAxiom extends LinearGuarded {

	private final AccessedRelation accessedRelation;

	/**
	 * Creates a defining axiom for this relation's attribute
	 * @param accessed
	 * @param attribute
	 */
	public DefiningAxiom(AccessedRelation accessed, Attribute attribute) {
		super(createLeft(accessed, attribute), createRight(accessed, attribute));
		this.accessedRelation = accessed;
	}

	/**
	 * Creates the left-hand side of a defining axiom for the input relation and attribute
	 * @param accessed
	 * @param attribute
	 * @return a conjunction of predicates corresponding to the LHS of a defining axiom
	 */
	private static PredicateFormula createLeft(AccessedRelation accessed, Attribute attribute) {
		return accessed.createAtoms();
	}

	/**
	 * Creates the right-hand side of a defining axiom for the input relation and attribute
	 * @param accessed
	 * @param attribute
	 * @return a conjunction of predicates corresponding to the RHS of a defining axiom
	 */
	private static Conjunction<PredicateFormula> createRight(AccessedRelation accessed, Attribute attribute) {
		List<PredicateFormula> rightAtoms = new ArrayList<>();
		Relation acc = AccessibleRelation.getInstance();
		rightAtoms.add(new PredicateFormula(acc, new Variable(attribute.getName())));
		return Conjunction.of(rightAtoms);

	}

	/**
	 * @return the accessed relation of the defining axiom
	 */
	public AccessedRelation getAccessedRelation() {
		return this.accessedRelation;
	}
}
