package uk.ac.ox.cs.pdq.planner.io.pretty;

import java.io.PrintStream;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang3.tuple.Pair;

import uk.ac.ox.cs.pdq.db.Constraint;
import uk.ac.ox.cs.pdq.fol.Constant;
import uk.ac.ox.cs.pdq.fol.Formula;
import uk.ac.ox.cs.pdq.fol.PredicateFormula;
import uk.ac.ox.cs.pdq.fol.Variable;
import uk.ac.ox.cs.pdq.io.Writer;
import uk.ac.ox.cs.pdq.io.pretty.PrettyFormulaWriter;
import uk.ac.ox.cs.pdq.io.pretty.PrettyWriter;
import uk.ac.ox.cs.pdq.planner.db.access.AccessibilityAxiom;
import uk.ac.ox.cs.pdq.planner.db.access.AccessibleSchema;
import uk.ac.ox.cs.pdq.planner.db.access.InferredAccessibleAxiom;
import uk.ac.ox.cs.pdq.planner.db.access.AccessibleSchema.AccessibleRelation;
import uk.ac.ox.cs.pdq.proof.Proof;
import uk.ac.ox.cs.pdq.proof.Proof.State;

import com.google.common.base.Joiner;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;

/**
 * Writes a concise representation of a query to the given output
 * 
 * @author Julien Leblay
 *
 */
public class ExtendedPrettyProofWriter extends PrettyWriter<Proof> implements Writer<Proof> {

	/**
	 * The default out to which queries should be written, if not 
	 * explicitly provided at write time.
	 */
	private PrintStream out;
	
	private final AccessibleSchema accSchema;

	/**
	 * 
	 * @param out the default output
	 * @param accSchema AccessibleSchema
	 */
	private ExtendedPrettyProofWriter(PrintStream out, AccessibleSchema accSchema) {
		this.out = out;
		this.accSchema = accSchema;
	}
	
	/**
	 * Fluent pretty writer provider.
	 * @param out
	 * @param accSchema AccessibleSchema
	 * @return a new PrettyQueryWriter with the given default output.
	 */
	public static ExtendedPrettyProofWriter to(PrintStream out, AccessibleSchema accSchema) {
		return new ExtendedPrettyProofWriter(out, accSchema);
	}
	
	/**
	 * @param out PrintStream
	 * @param p Proof
	 */
	@Override
	public void write(PrintStream out, Proof p) {
		String sep = "";
		for (State state: p.getStates()) {
			AccessibilityAxiom axiom = state.getAxiom();
			out.print("Axiom: ");
			out.println(axiom.getBaseRelation().getName() + '/' + axiom.getAccessMethod().getName());
			
			
			PrettyFormulaWriter<Formula> dw = PrettyFormulaWriter.to(out).indented();
			out.print("Groundings: ");
			for (Map<Variable, Constant> candidate: state.getMatches()) {
				out.print('\n');
				dw.write(axiom.ground(candidate));
				InferredAccessibleAxiom infAcc = this.accSchema.getInferredAccAxiom(axiom.getBaseRelation());
				out.print('\n');
				dw.write(infAcc.ground(candidate));

//				System.out.println("InfAcc1: " + infAcc);
//
//				for (InferredAccessibleAxiom ifa: this.accSchema.getShortInferredAccessibilityAxioms()) {
//					System.out.println("InfAcc2: " + ifa);
//					for (PredicateFormula pred: ifa.getLeft().getSubFormula()) {
//						if (pred.getSignature().equals(this.accSchema.getInferredAccessibleRelation(axiom.getBaseRelation()))) {
//							System.out.println("InfAcc3: " + ifa);
////							List<Term> terms = ifa.getLeft().getTerms();
////							if (candidate.keySet().containsAll(terms)) {
//								out.print("\n");
//								dw.write(ifa.ground(candidate));
////							}
//						}
//					}
//				}
				out.println();
			}
			out.println();
		}
	}
	
	/**
	 * @param proof Proof
	 * @param target Iterable<? extends PredicateFormula>
	 * @return String
	 */
	public static String traceBack(Proof proof, Iterable<? extends PredicateFormula> target) {
		StringBuilder result = new StringBuilder();
		Set<PredicateFormula> todo = Sets.newLinkedHashSet(target);
//		Set<Collection<PredicateFormula>> done = Sets.newLinkedHashSet();
		Map<PredicateFormula, Pair<Constraint, HashSet<PredicateFormula>>> provenance = null;
//		Map<PredicateFormula, Pair<IC, HashSet<PredicateFormula>>> provenance = proof.getAtomProvenance();
//System.out.println("Provenance " + provenance);
		while (!todo.isEmpty()) {
			for (PredicateFormula p: Lists.newArrayList(todo)) {
//System.out.println("Checking " + p);
				Pair<Constraint, HashSet<PredicateFormula>> rule = provenance.get(p);
				if (rule != null) {
//					if (!done.contains(rule.getRight())) {
						String s = Joiner.on(" & ").join(rule.getRight()) + " -> " + p + "..." + '\n';
						result.insert(0, s);
						for (PredicateFormula q: rule.getRight()) {
							if (!q.getSignature().equals(AccessibleRelation.getInstance())) {
								todo.add(q);
							}
						}
//					}
//					done.add(rule.getRight());
				}
				todo.remove(p);
			}
		}
		return result.toString();
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.builder.io.PrettyWriter#write(java.lang.Object)
	 */
	/**
	 * @param q Proof
	 */
	@Override
	public void write(Proof q) {
		this.write(this.out, q);
	}
}
