package uk.ac.ox.cs.pdq.planner.linear.cost;

import org.apache.log4j.Logger;

import uk.ac.ox.cs.pdq.cost.estimators.AccessCountCostEstimator;
import uk.ac.ox.cs.pdq.cost.estimators.SimpleCostEstimator;
import uk.ac.ox.cs.pdq.logging.performance.StatisticsCollector;
import uk.ac.ox.cs.pdq.plan.LinearPlan;

import com.google.common.eventbus.EventBus;

/**
 * A simple cost estimator.
 * According to this implementation, the cost of a plan equals the sum of the costs of the 
 * associated access commands.
 * 
 * Each access command must maintain a non-null binding object
 * (see uk.ac.ox.cs.pdq.schema.Binding) which, in turn, must have a non-null cost
 * (encapsulated in an object BindingStatsImplementation).
 * 
 * If the above prerequisites are not satisfied, then a runtime exception is 
 * thrown.
 * 
 * @author Efthymia Tsamoura 
 * 
 */
public class LinearAccessCountCostEstimator extends AccessCountCostEstimator<LinearPlan> implements Propagatable<LinearPlan> {

	/** Logger */
	protected static Logger log = Logger.getLogger(LinearAccessCountCostEstimator.class);

	/**
	 * Default constructor
	 * @param eventBus
	 * @param collectStats
	 */
	public LinearAccessCountCostEstimator() {
		super(null, false);
	}

	/**
	 * Default constructor
	 * @param eventBus
	 * @param collectStats
	 */
	public LinearAccessCountCostEstimator(EventBus eventBus, boolean collectStats) {
		super(eventBus, collectStats);
	}

	/**
	 * Default constructor
	 * @param eventBus
	 * @param collectStats
	 */
	public LinearAccessCountCostEstimator(StatisticsCollector stats) {
		super(stats);
	}

	@Override
	public LinearAccessCountCostEstimator clone() {
		return new LinearAccessCountCostEstimator(this.stats.clone());
	}

	@Override
	public SimplePropagator getPropagator() {
		return new SimplePropagator((SimpleCostEstimator<LinearPlan>) this);
	}

}
