package uk.ac.ox.cs.pdq.planner.linear.cost;

import java.util.Properties;

import org.apache.log4j.Logger;

import uk.ac.ox.cs.pdq.cost.CostParameters.BlackBoxQueryTypes;
import uk.ac.ox.cs.pdq.cost.estimators.BlackBoxCostEstimator;
import uk.ac.ox.cs.pdq.cost.estimators.PostgresqlBlackBoxEstimator;
import uk.ac.ox.cs.pdq.logging.performance.StatisticsCollector;
import uk.ac.ox.cs.pdq.plan.LinearPlan;

import com.google.common.eventbus.EventBus;

/**
 * A simple cost estimator.
 * According to this implementation, the cost of a plan equals the sum of the costs of the 
 * associated access commands.
 * 
 * Each access command must maintain a non-null binding object
 * (see uk.ac.ox.cs.pdq.schema.Binding) which, in turn, must have a non-null cost
 * (encapsulated in an object BindingStatsImplementation).
 * 
 * If the above prerequisites are not satisfied, then a runtime exception is 
 * thrown.
 * 
 * @author Efthymia Tsamoura 
 * 
 */
public class LinearPostgresqlBlackBoxEstimator extends PostgresqlBlackBoxEstimator<LinearPlan> implements Propagatable<LinearPlan> {

	/** Logger */
	protected static Logger log = Logger.getLogger(LinearPostgresqlBlackBoxEstimator.class);

	/**
	 * Default constructor
	 * @param eventBus
	 * @param collectStats
	 */
	public LinearPostgresqlBlackBoxEstimator(EventBus eventBus, boolean collectStats, Properties prop) {
		super(eventBus, collectStats, prop);
	}

	/**
	 * Default constructor
	 * @param eventBus
	 * @param collectStats
	 */
	public LinearPostgresqlBlackBoxEstimator(EventBus eventBus, boolean collectStats, Properties prop, BlackBoxQueryTypes type) {
		super(eventBus, collectStats, prop, type);
	}
	
	public LinearPostgresqlBlackBoxEstimator(StatisticsCollector stats, Properties prop, BlackBoxQueryTypes type) {
		super(stats, prop, type);
	}

	@Override
	public LinearPostgresqlBlackBoxEstimator clone() {
		return new LinearPostgresqlBlackBoxEstimator(this.stats.clone(), this.properties, this.queryType);
	}

	@Override
	public BlackBoxPropagator getPropagator() {
		return new BlackBoxPropagator((BlackBoxCostEstimator<LinearPlan>) this);
	}

}
