package uk.ac.ox.cs.pdq.planner.linear.cost;

import org.apache.log4j.Logger;

import uk.ac.ox.cs.pdq.cost.estimators.BlackBoxCostEstimator;
import uk.ac.ox.cs.pdq.cost.estimators.WhiteBoxCostEstimator;
import uk.ac.ox.cs.pdq.cost.statistics.estimators.CardinalityEstimator;
import uk.ac.ox.cs.pdq.logging.performance.StatisticsCollector;
import uk.ac.ox.cs.pdq.plan.LinearPlan;

import com.google.common.eventbus.EventBus;

/**
 * A simple cost estimator.
 * According to this implementation, the cost of a plan equals the sum of the costs of the 
 * associated access commands.
 * 
 * Each access command must maintain a non-null binding object
 * (see uk.ac.ox.cs.pdq.schema.Binding) which, in turn, must have a non-null cost
 * (encapsulated in an object BindingStatsImplementation).
 * 
 * If the above prerequisites are not satisfied, then a runtime exception is 
 * thrown.
 * 
 * @author Efthymia Tsamoura 
 * 
 */
public class LinearWhiteBoxCostEstimator extends WhiteBoxCostEstimator<LinearPlan> implements Propagatable<LinearPlan> {

	/** Logger */
	protected static Logger log = Logger.getLogger(LinearWhiteBoxCostEstimator.class);

	/**
	 * Default constructor
	 * @param eventBus
	 * @param collectStats
	 */
	public LinearWhiteBoxCostEstimator(EventBus eventBus, boolean collectStats, CardinalityEstimator ce) {
		super(eventBus, collectStats, ce);
	}

	/**
	 * Default constructor
	 * @param eventBus
	 * @param collectStats
	 */
	public LinearWhiteBoxCostEstimator(StatisticsCollector stats, CardinalityEstimator ce) {
		super(stats, ce);
	}

	@Override
	public LinearWhiteBoxCostEstimator clone() {
		return new LinearWhiteBoxCostEstimator(this.stats.clone(), this.cardEstimator.clone());
	}

	@Override
	public BlackBoxPropagator getPropagator() {
		return new BlackBoxPropagator((BlackBoxCostEstimator<LinearPlan>) this);
	}

}
