package uk.ac.ox.cs.pdq.planner.linear.node;

import java.util.Set;

import uk.ac.ox.cs.pdq.cost.estimators.SimpleCostEstimator;
import uk.ac.ox.cs.pdq.plan.LinearPlan;
import uk.ac.ox.cs.pdq.planner.PlannerException;
import uk.ac.ox.cs.pdq.planner.explorer.ConfigurationFactory;
import uk.ac.ox.cs.pdq.planner.linear.LinearChaseConfiguration;
import uk.ac.ox.cs.pdq.planner.linear.explorer.Candidate;
import uk.ac.ox.cs.pdq.planner.reasoning.chase.state.ChaseState;

import com.google.common.base.Preconditions;

/**
 * Creates nodes based on the input arguments
 *
 * @author Efthymia Tsamoura
 */
public final class NodeFactory<S extends ChaseState, N extends SearchNode> {

	private final ConfigurationFactory<S, LinearPlan> configurationFactory;

	/**
	 * Constructor for NodeFactory.
	 * @param configurationFactory ConfigurationFactory<S,LinearPlan>
	 */
	public NodeFactory(ConfigurationFactory<S, LinearPlan> configurationFactory) {
		Preconditions.checkArgument(configurationFactory != null);
		this.configurationFactory = configurationFactory;
	}

	/**
	 * @param configuration LinearConfiguration
	 * @return a node with the input configuration * @throws PlannerException
	 */
	public N getInstance(LinearChaseConfiguration configuration) throws PlannerException {
		if (this.configurationFactory.getCostEstimator() instanceof SimpleCostEstimator) {
			return (N) new SimpleNode(configuration);
		}
		return (N) new BlackBoxNode(configuration);
	}

	/**
	 * @param parentNode
	 * @param exposedCandidates
	 * @return a node having the given parent node and the given exposed candidate facts
	 * @throws PlannerException
	 */
	public N getInstance(N parentNode, Set<Candidate> exposedCandidates) throws PlannerException {
		LinearChaseConfiguration configuration = this.configurationFactory.getLinearInstance(parentNode.getConfiguration(), exposedCandidates);
		if (parentNode instanceof SimpleNode) {
			return (N) new SimpleNode((SimpleNode) parentNode, configuration);
		}
		return (N) new BlackBoxNode((BlackBoxNode) parentNode, configuration);
	}
}