package uk.ac.ox.cs.pdq.planner.parallel;

import java.util.List;
import java.util.Set;

import uk.ac.ox.cs.pdq.planner.dag.DAGChaseConfiguration;
import uk.ac.ox.cs.pdq.planner.reasoning.chase.state.ChaseState;

/**
 * Returns the results of a call to IterativeExecutor.finalIteration
 *
 * @author Efthymia Tsamoura
 *
 * @param <S>
 */
public class FinalIterationThreadResults<S extends ChaseState> {
	private final List<DAGChaseConfiguration<S>> output;
	private final Set<DAGChaseConfiguration<S>> successful;
	private final DAGChaseConfiguration<S> best;

	/**
	 * Constructor for FinalIterationThreadResults.
	 * @param output List<DAGChaseConfiguration<S>>
	 * @param successfulConfigurations Set<DAGChaseConfiguration<S>>
	 * @param bestConfiguration DAGChaseConfiguration<S>
	 */
	public FinalIterationThreadResults(List<DAGChaseConfiguration<S>> output,
			Set<DAGChaseConfiguration<S>> successfulConfigurations,
			DAGChaseConfiguration<S> bestConfiguration) {
		this.output = output;
		this.best = bestConfiguration;
		this.successful = successfulConfigurations;
	}

	/**
	 * @return List<DAGChaseConfiguration<S>>
	 */
	public List<DAGChaseConfiguration<S>> getOutput() {
		return this.output;
	}

	/**
	 * @return DAGChaseConfiguration<S>
	 */
	public DAGChaseConfiguration<S> getBest() {
		return this.best;
	}

	/**
	 * @return Set<DAGChaseConfiguration<S>>
	 */
	public Set<DAGChaseConfiguration<S>> getSuccessful() {
		return this.successful;
	}
}