package uk.ac.ox.cs.pdq.planner.parallel;

import uk.ac.ox.cs.pdq.planner.PlannerParameters.IterativeExecutorTypes;
import uk.ac.ox.cs.pdq.planner.dag.cost.ConfigurationCostEstimator;
import uk.ac.ox.cs.pdq.planner.homomorphism.HomomorphismDetector;
import uk.ac.ox.cs.pdq.planner.reasoning.chase.ReasonerFactory;
import uk.ac.ox.cs.pdq.planner.reasoning.chase.dominance.SuccessDominance;
import uk.ac.ox.cs.pdq.planner.reasoning.chase.state.ChaseState;

import com.google.common.eventbus.EventBus;

/**
 * Creates IterativeExecutor objects
 *
 * @author Efthymia Tsamoura
 */
public class IterativeExecutorFactory {
	/**
	 * @param executorType IterativeExecutorTypes
	 * @param parallelThreads int
	 * @param reasonerFactory ReasonerFactory
	 * @param detector HomomorphismDetector
	 * @param estimator CostEstimator<DAGPlan>
	 * @param successDominanceDetector SuccessDominance
	 * @param eventBus EventBus
	 * @param collectStats boolean
	 * @return IterativeExecutor<S>
	 * @throws Exception
	 */
	public static <S extends ChaseState> IterativeExecutor<S> createIterativeExecutor (
			IterativeExecutorTypes executorType,
			int parallelThreads,
			ReasonerFactory reasonerFactory,
			HomomorphismDetector detector,
			ConfigurationCostEstimator estimator,
			SuccessDominance successDominanceDetector,
			EventBus eventBus,
			boolean collectStats) throws Exception{
		switch(executorType) {
		case MULTITHREADED:
			MultiThreadedContext mtcontext = new MultiThreadedContext(parallelThreads,
					reasonerFactory,
					detector,
					estimator,
					successDominanceDetector,
					eventBus, collectStats);
			MultiThreadedExecutor<S> executor = new MultiThreadedExecutor<>(mtcontext);
			return executor;
		default:
			throw new java.lang.IllegalArgumentException();
		}
	}
}
