package uk.ac.ox.cs.pdq.planner.reasoning.chase;

import java.util.Collection;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang3.tuple.Pair;
import org.jgrapht.Graph;
import org.jgrapht.graph.DefaultEdge;

import uk.ac.ox.cs.pdq.db.Constraint;
import uk.ac.ox.cs.pdq.fol.PredicateFormula;
import uk.ac.ox.cs.pdq.planner.dag.DAGConfiguration;
import uk.ac.ox.cs.pdq.planner.reasoning.chase.state.ChaseState;

/**
 * Maintains information w.r.t. chasing plus answers proximity queries
 *
 * @author Efthymia Tsamoura
 *
 */
public interface FiringGraph<S extends ChaseState> {
	
	/**
	 * Updates the internal structures based given the input chasing information
	 * @param dependency
	 * @param sources
	 * 			the facts that where used to ground the input dependency
	 * @param targets
	 * 			the consequence facts
	 * @param initialState
	 * 		true if we are in the initial configuration
	 */
	void put(Constraint dependency, Collection<PredicateFormula> sources, Collection<PredicateFormula> targets, boolean initialState);

	/**
	 * @param dependency IC
	 * @param source PredicateFormula
	 * @param target PredicateFormula
	 * @param initialState boolean
	 */
	void put(Constraint dependency, PredicateFormula source, PredicateFormula target, boolean initialState);

	/**
	 *
	 * @param fact
	 * @return
	 * 		the history of the input fact, i.e., the firings that leaded to the creation of this fact
	 */
	Pair<Constraint, Collection<PredicateFormula>> getFactProvenance(PredicateFormula fact);

	/**
	 *
	 * @return
	 * 		the history of the chase facts, i.e., the firings that leaded to the creation of each fact
	 */
	Map<PredicateFormula, Pair<Constraint, Collection<PredicateFormula>>> getFactProvenance();

	/**
	 *
	 * @param dependency
	 * @param facts
	 * @return
	 * 		true if the input dependency has been already fired given the input facts
	 */
	boolean isFired(Constraint dependency, Collection<PredicateFormula> facts);

	/**
	 *
	 * @return
	 * 		the chase graph
	 */
	Graph<PredicateFormula, DefaultEdge> getGraph();

	/**
	 * @return FiringGraph<S>
	 */
	FiringGraph<S> clone();

	/**
	 * Merges two FiringGraph objects
	 * @param source
	 *
	 * @return FiringGraph<S>
	 */
	FiringGraph<S> merge(FiringGraph<S> source);
	
	Set<Collection<PredicateFormula>> getPreconditions();

	public Collection<Collection<PredicateFormula>> getConsequences(Collection<PredicateFormula> key);
}
