package uk.ac.ox.cs.pdq.planner.reasoning.chase;

import java.util.LinkedHashMap;
import java.util.Map;

import uk.ac.ox.cs.pdq.fol.Constant;
import uk.ac.ox.cs.pdq.fol.PredicateFormula;
import uk.ac.ox.cs.pdq.fol.Variable;
import uk.ac.ox.cs.pdq.planner.db.access.AccessibilityAxiom;
import uk.ac.ox.cs.pdq.reasoning.Match;

/**
 * Creates matching objects
 *
 * @author Efthymia Tsamoura
 */
public class MatchFactory {

	/**
	 *
	 * @param axiom
	 * @param fact
	 * @return
	 * 		a matching given the input accessibility axiom and the input fact
	 */
	public static Match getMatching(AccessibilityAxiom axiom, PredicateFormula fact) {
		Map<Variable, Constant> map = createMap(axiom, fact);
		if(fact instanceof BagBoundPredicate) {
			return new BagMatch(axiom, map, ((BagBoundPredicate) fact).getBag());
		}
		return new Match(axiom, map);
	}

	/**
	 * @param axiom AccessibilityAxiom
	 * @param fact PredicateFormula
	 * @return Map<Variable, Constant>
	 */
	private static Map<Variable, Constant> createMap(AccessibilityAxiom axiom, PredicateFormula fact) {
		Map<Variable, Constant> map = new LinkedHashMap<>();
		int i = 0;
		for(Variable variable:axiom.getGuard().getVariables()) {
			map.put(variable, (Constant) fact.getTerm(i));
			i++;
		}
		return map;
	}
}
