package uk.ac.ox.cs.pdq.planner.reasoning.chase.equivalence;

import java.util.Set;

import com.google.common.collect.Sets;

import uk.ac.ox.cs.pdq.fol.Constant;
import uk.ac.ox.cs.pdq.planner.reasoning.chase.configuration.ChaseConfiguration;


/**
 * Performs fast equivalence checks.
 * According to this implementation two configurations c and c' are equivalent if the have the same inferred accessible facts. 
 * In order to perform this kind of check Skolem constants must be assigned to formula variables during chasing.
 *
 * @author Efthymia Tsamoura
 */
public class FastFactEquivalence implements FactEquivalence{

	/**
	 * @param source ChaseConfiguration
	 * @param target ChaseConfiguration
	 * @return true if source and target configurations are equivalent
	 */
	@Override
	public boolean isEquivalent(ChaseConfiguration source, ChaseConfiguration target) {
		Set<Constant> inputs1 = Sets.newLinkedHashSet(source.getInput());
		Set<Constant> inputs2 = Sets.newLinkedHashSet(target.getInput());
		if(inputs1.equals(inputs2)) {
			if (source.getInferred().equals(target.getInferred())) {
				return true;
			}
		}
		return false;
	}

	/**
	 * @return FastFactDominance
	 */
	@Override
	public FastFactEquivalence clone() {
		return new FastFactEquivalence();
	}
}