package uk.ac.ox.cs.pdq.planner.reasoning.chase.state;

import java.util.Collection;

import uk.ac.ox.cs.pdq.fol.PredicateFormula;
import uk.ac.ox.cs.pdq.planner.homomorphism.DBHomomorphismManager;
import uk.ac.ox.cs.pdq.planner.homomorphism.HomomorphismDetector;
import uk.ac.ox.cs.pdq.planner.reasoning.chase.BagFactory;


/**
 * Creates a configuration states using the input arguments
 *
 * @author Efthymia Tsamoura
 */
public final class StateFactory {

	/**
	 *
	 * @param facts
	 * 		Input facts
	 * @param detector HomomorphismDetector
	 * @return
	 * 		A configuration state initialised using the input facts.
	 * 		If the facts must be organised into a tree of bags, then a tree
	 *      state with a single root bag is created.
	 * 		Otherwise, a new list state is created.
	 */
	public static <S extends ChaseState> S getInstance(HomomorphismDetector detector, Collection<PredicateFormula> facts) {
		return getInstance(null, detector, facts);
	}

	/**
	 *
	 * @param facts
	 * 		Input facts
	 * @param bagFactory BagFactory
	 * @param detector HomomorphismDetector
	 * @return
	 * 		A configuration state initialised using the input facts.
	 * 		If the facts must be organised into a tree of bags, 
	 * 		then a tree state with a single root bag is created.
	 * 		Otherwise, a new list state is created.
	 */
	public static <S extends ChaseState> S getInstance(BagFactory bagFactory, HomomorphismDetector detector, Collection<PredicateFormula> facts) {
		if (bagFactory != null) {
			return (S) new DatabaseTreeState((DBHomomorphismManager) detector, facts, bagFactory);
		}
		return (S) new DatabaseListState((DBHomomorphismManager) detector, facts);
	}
}
