package uk.ac.ox.cs.pdq.test.planner.db.access;

import java.util.Random;

import org.junit.Assert;
import org.junit.Test;

import uk.ac.ox.cs.pdq.db.Attribute;
import uk.ac.ox.cs.pdq.db.Relation;
import uk.ac.ox.cs.pdq.db.TypedConstant;
import uk.ac.ox.cs.pdq.fol.PredicateFormula;
import uk.ac.ox.cs.pdq.fol.Skolem;
import uk.ac.ox.cs.pdq.planner.db.access.AccessibleSchema.AccessedRelation;
import uk.ac.ox.cs.pdq.planner.db.access.AccessibleSchema.AccessibleRelation;
import uk.ac.ox.cs.pdq.planner.db.access.AccessibleSchema.InferredAccessibleRelation;

import com.google.common.collect.Lists;

public class PredicateFormulaTest {

	private Random random = new Random();

	@Test public void testIsSchemaFact() {
		Relation r = new Relation("r", Lists.newArrayList(
				new Attribute(String.class, "a1"), new Attribute(String.class, "a2"))) {};
				TypedConstant<String> c = new TypedConstant<>("c");
				Skolem s = new Skolem("s");
				Assert.assertFalse("PredicateFormula must not be of type Accessed",
						new PredicateFormula(r, Lists.newArrayList(s, c)).getSignature() instanceof AccessedRelation);
				Assert.assertFalse("PredicateFormula must not be of type Accessible",
						new PredicateFormula(r, Lists.newArrayList(s, c)).getSignature() instanceof AccessedRelation);
				Assert.assertFalse("PredicateFormula must not be of type InferredAccessible",
						new PredicateFormula(r, Lists.newArrayList(s, c)).getSignature() instanceof InferredAccessibleRelation);
				PredicateFormula p = new PredicateFormula(r, Lists.newArrayList(s, c));
				boolean isSchemaFact = !(p.getSignature() instanceof AccessedRelation) &&
						!(p.getSignature() instanceof AccessibleRelation) &&
						!(p.getSignature() instanceof InferredAccessibleRelation);
				Assert.assertTrue("PredicateFormula must not be an schema fact ", isSchemaFact);
	}

	@Test public void testIsAccessibleFact() {
		Relation r = AccessibleRelation.getInstance();
		Skolem s = new Skolem("s");
		Assert.assertTrue("PredicateFormula must be of type Accessible",
				new PredicateFormula(r, Lists.newArrayList(s)).getSignature() instanceof AccessibleRelation);
	}

	@Test public void testIsInferredAccessibleFact() {
		Relation r = new InferredAccessibleRelation(
				new Relation("r", Lists.newArrayList(
						new Attribute(String.class, "a1"),
						new Attribute(String.class, "a2"))) {});
		TypedConstant<String> c = new TypedConstant<>("c");
		Skolem s = new Skolem("s");
		Assert.assertTrue("PredicateFormula must be of type InferredAccessible",
				new PredicateFormula(r, Lists.newArrayList(s, c)).getSignature() instanceof InferredAccessibleRelation);
	}

	@Test public void testIsAccessedFact() {
		Relation r = new AccessedRelation(new Relation("r", Lists.newArrayList(
				new Attribute(String.class, "a1"), new Attribute(String.class, "a2"))) {});
		TypedConstant<String> c = new TypedConstant<>("c");
		Skolem s = new Skolem("s");
		Assert.assertTrue("PredicateFormula must be of type Accessed",
				new PredicateFormula(r, Lists.newArrayList(s, c)).getSignature() instanceof AccessedRelation);
	}

}
