package uk.ac.ox.cs.pdq.test.planner.reasoning.chase;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import com.google.common.base.Joiner;
import com.google.common.eventbus.EventBus;

import uk.ac.ox.cs.pdq.db.Schema;
import uk.ac.ox.cs.pdq.db.TypedConstant;
import uk.ac.ox.cs.pdq.fol.ConjunctiveQuery;
import uk.ac.ox.cs.pdq.fol.PredicateFormula;
import uk.ac.ox.cs.pdq.fol.Query;
import uk.ac.ox.cs.pdq.fol.Signature;
import uk.ac.ox.cs.pdq.fol.Skolem;
import uk.ac.ox.cs.pdq.fol.Term;
import uk.ac.ox.cs.pdq.fol.Variable;
import uk.ac.ox.cs.pdq.io.xml.QueryReader;
import uk.ac.ox.cs.pdq.io.xml.SchemaReader;
import uk.ac.ox.cs.pdq.planner.PlannerParameters.HomomorphismDetectorTypes;
import uk.ac.ox.cs.pdq.planner.db.access.AccessibleSchema;
import uk.ac.ox.cs.pdq.planner.homomorphism.HomomorphismDetector;
import uk.ac.ox.cs.pdq.planner.homomorphism.HomomorphismManagerFactory;
import uk.ac.ox.cs.pdq.planner.reasoning.chase.RestrictedChaser;
import uk.ac.ox.cs.pdq.planner.reasoning.chase.state.DatabaseListState;

/**
 * Tests the restricted chase implementation
 * @author Efthymia Tsamoura
 *
 */
public class RestrictedChaserTest {

	private EventBus eventBus = new EventBus();
	
	private static String PATH = "data/test_restricted_chaser/";
	String[] schemata1 = {
			"schema_fk_view.xml",
			"schema_fk_view.xml",
			"schema_fk_view.xml",
			"schema_fk_view.xml",
			"schema_fk_view.xml",
			"schema_fk_view.xml",
			"schema_fk_view.xml",
			"schema_fk_view.xml",
			"schema_fk_view.xml",
			"schema_fk_view.xml"
			};
	
	String[] schemata2 = {
			"schema_demo.xml",
			"schema_demo.xml",
			"schema_bio.xml",
			"schema_bio.xml",
			"schema_bio.xml",
			"schema_bio.xml",
			"schema_bio.xml",
			"schema_bio.xml",
			"schema_bio.xml",
			"schema_bio.xml",
			"schema_bio.xml",
			"schema_bio.xml",
			"schema_bio.xml",
			"schema_bio.xml",
			"schema_bio.xml",
			"schema_bio.xml",
			"schema_bio.xml"
			};
	String[] queries1 = {
			"query_fk_view_1.xml",
			"query_fk_view_2.xml",
			"query_fk_view_3.xml",
			"query_fk_view_4.xml",
			"query_fk_view_5.xml",
			"query_fk_view_6.xml",
			"query_fk_view_7.xml",
			"query_fk_view_8.xml",
			"query_fk_view_9.xml",
			"query_fk_view_10.xml"
			};
	String[] queries2 = {
			"query_demo_1.xml",
			"query_demo_2.xml",
			"query_bio_1.xml",
			"query_bio_2.xml",
			"query_bio_3.xml",
			"query_bio_4.xml",
			"query_bio_5.xml",
			"query_bio_6.xml",
			"query_bio_7.xml",
			"query_bio_8.xml",
			"query_bio_9.xml",
			"query_bio_10.xml",
			"query_bio_11.xml",
			"query_bio_12.xml",
			"query_bio_13.xml",
			"query_bio_14.xml",
			"query_bio_15.xml"
			};
	String[] facts1 = {
			"facts_fk_view_1.txt",
			"facts_fk_view_2.txt",
			"facts_fk_view_3.txt",
			"facts_fk_view_4.txt",
			"facts_fk_view_5.txt",
			"facts_fk_view_6.txt",
			"facts_fk_view_7.txt",
			"facts_fk_view_8.txt",
			"facts_fk_view_9.txt",
			"facts_fk_view_10.txt"
			};
	
	String[] facts2 = {
			"facts_demo_1.txt",
			"facts_demo_2.txt",
			"facts_bio_1.txt",
			"facts_bio_2.txt",
			"facts_bio_3.txt",
			"facts_bio_4.txt",
			"facts_bio_5.txt",
			"facts_bio_6.txt",
			"facts_bio_7.txt",
			"facts_bio_8.txt",
			"facts_bio_9.txt",
			"facts_bio_10.txt",
			"facts_bio_11.txt",
			"facts_bio_12.txt",
			"facts_bio_13.txt",
			"facts_bio_14.txt",
			"facts_bio_15.txt"
			};
	
	boolean canonicalNames = true;
	String driver = null;
	String url = "jdbc:mysql://localhost/";
	String database = "pdq_chase";
	String username = "root";
	String password ="root";

	@Before
	public void prepare() {

	}
	
	@Test
	public void test1() {
		this.test(this.schemata1, this.queries1, this.facts1);
	}
	
	@Test
	public void test2() {
		this.test(this.schemata2, this.queries2, this.facts2);
	}


	public void test(String[] schemata, String[] queries, String[] facts) {
		for(int i = 0; i < schemata.length; ++i) {
			String s = schemata[i];
			String q = queries[i];
			String f = facts[i];

			try(FileInputStream sis = new FileInputStream(PATH + s);
					FileInputStream qis = new FileInputStream(PATH + q)) {

				Schema schema = new SchemaReader().read(sis);
				ConjunctiveQuery query = new QueryReader(schema).read(qis);
				
				if (schema == null || query == null) {
					throw new IllegalStateException("Schema and query must be provided.");
				}

				schema.updateConstantsMap(query.getSchemaConstants());
				AccessibleSchema accessibleSchema = new AccessibleSchema(schema);
				Query<?> accessibleQuery = accessibleSchema.accessible(query, query.canonicalMapping());

				Collection<PredicateFormula> expected = loadFacts(PATH + f, accessibleSchema);
				

				RestrictedChaser reasoner = new RestrictedChaser(
						this.eventBus,
						false,
						this.canonicalNames);

				HomomorphismDetector detector =
						HomomorphismManagerFactory.getInstance(accessibleSchema, accessibleQuery, 
								HomomorphismDetectorTypes.DATABASE,
								this.driver,
								this.url,
								this.database,
								this.username,			
								this.password);

				DatabaseListState state = reasoner.initialize(detector, query, schema);
				reasoner.chase(state, schema.getDependencies());
				
				Assert.assertEquals(state.getFacts(), expected);

			} catch (FileNotFoundException e) {
				System.out.println("Cannot find input files");
			} catch (Exception e) {
				System.out.println("EXCEPTION: " + e.getClass().getSimpleName() + " " + e.getMessage());
				e.printStackTrace();
			} catch (Error e) {
				System.out.println("ERROR: " + e.getClass().getSimpleName() + " " + e.getMessage());
				e.printStackTrace();
				System.exit(-1);
			}
		}
	}

	private Collection<PredicateFormula> loadFacts(String fileName, AccessibleSchema accessibleSchema) {
		Collection<PredicateFormula> predicates = new HashSet<>();
		String line = null;
		try {
			FileReader fileReader = new FileReader(fileName);
			BufferedReader bufferedReader = new BufferedReader(fileReader);
			while((line = bufferedReader.readLine()) != null) {
				predicates.add(this.loadFact(line, accessibleSchema));
			}
			bufferedReader.close();    
			return predicates;
		}
		catch(FileNotFoundException ex) {      
			ex.printStackTrace(System.out);
		}
		catch(IOException ex) {
			ex.printStackTrace(System.out);
		}
		return null;
	}

	private PredicateFormula loadFact(String predicate, AccessibleSchema accessibleSchema) {		
		
		int index1 = predicate.indexOf("(");
		int index2 = predicate.indexOf(")");
		
		String relationName = (String) predicate.subSequence(0, index1);
		Signature signature = accessibleSchema.getRelation(relationName);
		String terms = (String) predicate.subSequence(index1 + 1, index2);
		List<Term> variables = new ArrayList<>();
		for(String term:terms.split(",")) {
			TypedConstant<?> constant = accessibleSchema.getConstant(term);
			if(constant != null) {
				variables.add(new TypedConstant<>(constant));
			}
			else {
				variables.add(new Skolem(term));
			}
		}
		return new PredicateFormula(signature, variables);
	}


}