package uk.ac.ox.cs.pdq.runtime.exec.iterator;

import java.util.ArrayList;
import java.util.List;
import java.util.NoSuchElementException;

import uk.ac.ox.cs.pdq.util.Tuple;
import uk.ac.ox.cs.pdq.util.TupleType;

import com.google.common.base.Preconditions;

/**
 * Count counts the number of tuples in a result set and returns it.
 * 
 * @author Julien Leblay
 */
public class Count extends UnaryIterator {

	/** The current result's count. */
	private Integer nextResult = null;
	
	/**
	 * Instantiates a new operator.
	 * 
	 * @param child TupleIterator
	 */
	public Count(TupleIterator child) {
		super(TupleType.DefaultFactory.create(Integer.class), child);
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#deepCopy()
	 */
	@Override
	public Count deepCopy() {
		return new Count(this.child.deepCopy());
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.UnaryIterator#hasNext()
	 */
	@Override
	public boolean hasNext() {
		Preconditions.checkArgument(this.open);
		return this.nextResult == null;
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#next()
	 */
	@Override
	public Tuple next() {
		Preconditions.checkArgument(this.open);
		if (this.nextResult == null) {
			this.nextResult = 0;
			while (this.child.hasNext()) {
				this.nextResult++;
				this.child.next();
			}
			return this.getType().createTuple(this.nextResult);
		}
		throw new NoSuchElementException("End of operator reached.");
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.UnaryIterator#reset()
	 */
	@Override
	public void reset() {
		super.reset();
		this.nextResult = null;
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#getColumnsDisplay()
	 */
	@Override
	public List<String> getColumnsDisplay() {
		List<String> result = new ArrayList<>();
		StringBuilder builder = new StringBuilder("COUNT(");
		String sep = "";
		for (String s: super.getColumnsDisplay()) {
			builder.append(sep).append(s);
			sep = ",";
		}
		builder.append(')');
		result.add(builder.toString());
		return result;
	}
}
