package uk.ac.ox.cs.pdq.runtime.exec.iterator;

import java.util.LinkedHashSet;
import java.util.NoSuchElementException;
import java.util.Set;

import uk.ac.ox.cs.pdq.util.Tuple;

import com.google.common.base.Preconditions;

/**
 * Distinct removes duplicates in a result set.
 * 
 * @author Julien Leblay
 */
public class Distinct extends UnaryIterator {

	/** The iteratorCache */
	private Set<Tuple> cache = new LinkedHashSet<>();

	/** The next result to return */
	private Tuple nextResult = null;
	
	/**
	 * Instantiates a new operator.
	 * 
	 * @param child TupleIterator
	 */
	public Distinct(TupleIterator child) {
		super(child);
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#deepCopy()
	 */
	@Override
	public Distinct deepCopy() {
		return new Distinct(this.child.deepCopy());
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.UnaryIterator#open()
	 */
	@Override
	public void open() {
		super.open();
		this.nextTuple();
	}

	/**
	 * Prepares the next tuple to return. If the end of the operator is reached,
	 * nextTuple shall be null.
	 */
	private void nextTuple() {
		this.nextResult = null;
		do {
			if (this.child.hasNext()) {
				this.nextResult = this.child.next();
				
			} else {
				this.nextResult = null;
			}
		} while (this.cache.contains(this.nextResult));
		if (this.nextResult != null) {
			this.cache.add(this.nextResult);
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.UnaryIterator#hasNext()
	 */
	@Override
	public boolean hasNext() {
		Preconditions.checkArgument(this.open);
		return this.nextResult != null;
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#next()
	 */
	@Override
	public Tuple next() {
		Preconditions.checkArgument(this.open);
		if (this.nextResult == null) {
			throw new NoSuchElementException("End of operator reached.");
		}
		Tuple result = this.nextResult;
		this.nextTuple();
		return result;
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.UnaryIterator#reset()
	 */
	@Override
	public void reset() {
		super.reset();
		this.cache.clear();
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#interrupt()
	 */
	@Override
	public void interrupt() {
		this.nextResult = null;
	}
}
