package uk.ac.ox.cs.pdq.runtime.exec.iterator;

import java.util.LinkedList;
import java.util.NoSuchElementException;

import uk.ac.ox.cs.pdq.util.Tuple;
import uk.ac.ox.cs.pdq.util.TupleType;
import uk.ac.ox.cs.pdq.util.Typed;

import com.google.common.base.Preconditions;

/**
 * The empty iterator is used to model iterator of an empty set.
 * Its hasNext() method always return false, next() throws a 
 * NoSuchElementException and other method have basically no effect.
 * 
 * This is currently used to model the child of a access pipeline iterator.
 * 
 * @author Julien Leblay
 */
public class EmptyIterator extends TupleIterator {

	/**
	 * Instantiates a new memory scan.
	 * 
	 */
	public EmptyIterator() {
		super(TupleType.EmptyTupleType, new LinkedList<Typed>());
	}

	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		StringBuilder result = new StringBuilder();
		result.append(this.getClass().getSimpleName());
		return result.toString();
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#open()
	 */
	@Override
	public void open() {
		this.open = true;
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#close()
	 */
	@Override
	public void close() {
		// Nothing to do
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#reset()
	 */
	@Override
	public void reset() {
		// Nothing to do
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#hasNext()
	 */
	@Override
	public boolean hasNext() {
		Preconditions.checkState(this.open);
		return false;
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#next()
	 */
	@Override
	public Tuple next() {
		Preconditions.checkState(this.open);
		throw new NoSuchElementException();
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#deepCopy()
	 */
	@Override
	public EmptyIterator deepCopy() {
		return new EmptyIterator();
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#interrupt()
	 */
	@Override
	public void interrupt() {
		this.interrupted = true;
	}
}