package uk.ac.ox.cs.pdq.runtime.exec.iterator;

import java.util.Iterator;
import java.util.List;

import uk.ac.ox.cs.pdq.util.Tuple;
import uk.ac.ox.cs.pdq.util.Typed;

import com.google.common.base.Preconditions;

/**
 * OpenScan over an in-memory relation or simply a collection of tuples.
 * 
 * @author Julien Leblay
 */
public class MemoryScan extends TupleIterator {

	/** The next tuple to return. */
	protected Iterator<Tuple> tupleIterator = null;

	/** The next tuple to return. */
	protected Iterable<Tuple> data = null;

	/**
	 * Instantiates a new memory scan.
	 * @param columns List<Typed>
	 * @param data Iterable<Tuple>
	 */
	public MemoryScan(List<Typed> columns, Iterable<Tuple> data) {
		super(columns);
		this.data = data;
	}

	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		StringBuilder result = new StringBuilder();
		result.append(this.getClass().getSimpleName());
		return result.toString();
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#open()
	 */
	@Override
	public void open() {
		this.tupleIterator = this.data.iterator();
		this.open = true;
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#close()
	 */
	@Override
	public void close() {
		// Nothing to do
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#reset()
	 */
	@Override
	public void reset() {
		this.tupleIterator = this.data.iterator();
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#hasNext()
	 */
	@Override
	public boolean hasNext() {
		Preconditions.checkState(this.open);
		return !this.interrupted && this.tupleIterator.hasNext();
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#next()
	 */
	@Override
	public Tuple next() {
		Preconditions.checkState(this.open);
		return this.tupleIterator.next();
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#deepCopy()
	 */
	@Override
	public MemoryScan deepCopy() {
		return new MemoryScan(this.columns, this.data);
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#interrupt()
	 */
	@Override
	public void interrupt() {
		this.interrupted = true;
	}
}