package uk.ac.ox.cs.pdq.runtime.exec.iterator;

import uk.ac.ox.cs.pdq.db.wrappers.RelationAccessWrapper;
import uk.ac.ox.cs.pdq.util.ResetableIterator;
import uk.ac.ox.cs.pdq.util.Tuple;
import uk.ac.ox.cs.pdq.util.Typed;

import com.google.common.collect.Lists;

/**
 * Scan over a relation
 * 
 * @author Julien Leblay
 */
public class Scan extends TupleIterator {

	/** The next tuple to return. */
	protected ResetableIterator<Tuple> tupleIterator = null;

	/** The next tuple to return. */
	protected Iterable<Tuple> tuples = null;

	protected RelationAccessWrapper relation = null;	

	/**
	 * Instantiates a new join.
	 * 
	 * @param relation RelationAccessWrapper
	 */
	public Scan(RelationAccessWrapper relation) {
		super(Lists.<Typed>newArrayList(relation.getAttributes()));
		this.relation = relation;
	}

	/* (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#toString()
	 */
	@Override
	public String toString() {
		StringBuilder result = new StringBuilder();
		result.append(this.getClass().getSimpleName());
		if (this.relation != null) {
			result.append('(').append(this.relation.getName()).append(')');
		}
		return result.toString();
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#open()
	 */
	@Override
	public void open() {
		this.tupleIterator = this.relation.iterator();
		this.tupleIterator.open();
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#close()
	 */
	@Override
	public void close() {
		// Nothing to do
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#reset()
	 */
	@Override
	public void reset() {
		this.tupleIterator.reset();
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#hasNext()
	 */
	@Override
	public boolean hasNext() {
		return !this.interrupted && this.tupleIterator.hasNext();
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#next()
	 */
	@Override
	public Tuple next() {
		return this.tupleIterator.next();
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#deepCopy()
	 */
	@Override
	public Scan deepCopy() {
		return new Scan(this.relation);
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#interrupt()
	 */
	@Override
	public void interrupt() {
		this.interrupted = true;
	}
}