package uk.ac.ox.cs.pdq.runtime.exec.iterator;

import java.util.List;

import uk.ac.ox.cs.pdq.util.ResetableIterator;
import uk.ac.ox.cs.pdq.util.Tuple;
import uk.ac.ox.cs.pdq.util.Typed;

/**
 * SurrogateIterator iterates over a trivial Iterator<Tuple>
 * 
 * @author Julien Leblay
 */
public class SurrogateIterator extends TupleIterator {


	/** Iterator over the output tuples. */
	protected final ResetableIterator<Tuple> outputs;
	
	/**
	 * Instantiates a new join.
	 * 
	 * @param columns List<Typed>
	 * @param inner ResetableIterator<Tuple>
	 */
	public SurrogateIterator(List<Typed> columns, ResetableIterator<Tuple> inner) {
		super(columns);
		this.outputs = inner;
	}

	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		StringBuilder result = new StringBuilder();
		result.append(this.getClass().getSimpleName());
		result.append('(').append(this.outputs).append(')');
		return result.toString();
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.UnaryIterator#open()
	 */
	@Override
	public void open() {
		this.outputs.open();
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.UnaryIterator#reset()
	 */
	@Override
	public void reset() {
		this.outputs.reset();
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.UnaryIterator#hasNext()
	 */
	@Override
	public boolean hasNext() {
		return this.outputs.hasNext();
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#next()
	 */
	@Override
	public Tuple next() {
		return this.outputs.next();
	}
	
	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#deepCopy()
	 */
	@Override
	public SurrogateIterator deepCopy() {
		return new SurrogateIterator(this.getColumns(), this.outputs);
	}

	/**
	 * @see java.lang.AutoCloseable#close()
	 */
	@Override
	public void close() {
		// nothing to do
	}

	@Override
	public void interrupt() {
		this.interrupted = true;
	}
}