package uk.ac.ox.cs.pdq.runtime.exec.iterator;

import java.util.List;

import uk.ac.ox.cs.pdq.util.TupleType;
import uk.ac.ox.cs.pdq.util.Typed;

import com.google.common.eventbus.EventBus;


/**
 * Superclass to all unary logical operator.
 * Factorize the open, close, reset methods and the default constructor.
 * 
 * @author Julien LEBLAY
 */
public abstract class UnaryIterator extends TupleIterator {

	/** The sole child of the operator */
	protected TupleIterator child = null;
	
	/**
	 * Instantiates a new operator.
	 * 
	 * @param child TupleIterator
	 */
	public UnaryIterator(TupleIterator child) {
		this(child.getType(), child.getColumns(), child);
	}
	
	/**
	 * Instantiates a new operator.
	 * 
	 * @param typeOverride TupleType
	 * @param child TupleIterator
	 */
	protected UnaryIterator(TupleType typeOverride, TupleIterator child) {
		this(typeOverride, child.getColumns(), child);
	}
	
	/**
	 * Instantiates a new operator.
	 * 
	 * @param columns List<Typed>
	 * @param child TupleIterator
	 */
	protected UnaryIterator(List<Typed> columns, TupleIterator child) {
		this(TupleType.DefaultFactory.createFromTyped(columns), columns, child);
	}
	
	/**
	 * Instantiates a new operator.
	 * 
	 * @param typeOverride TupleType
	 * @param columns List<Typed>
	 * @param child TupleIterator
	 */
	protected UnaryIterator(TupleType typeOverride, List<Typed> columns, TupleIterator child) {
		super(typeOverride, columns);
		this.child = child;
	}

	/**
	 * Gets the child.
	 * 
	 * @return the child
	 */
	public TupleIterator getChild() {
		return this.child;
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#open()
	 */
	@Override
	public void open() {
		this.child.open();
		this.open = true;
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#close()
	 */
	@Override
	public void close() {
		this.child.close();
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#reset()
	 */
	@Override
	public void reset() {
		this.child.reset();
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#setEventBus(com.google.common.eventbus.EventBus)
	 */
	@Override
	public void setEventBus(EventBus eb) {
		super.setEventBus(eb);
		this.child.setEventBus(eb);
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#interrupt()
	 */
	@Override
	public void interrupt() {
		this.interrupted = true;
		this.child.interrupt();
	}
}
