package uk.ac.ox.cs.pdq.runtime.exec.iterator.open;

import java.util.ArrayList;
import java.util.List;
import java.util.NoSuchElementException;

import uk.ac.ox.cs.pdq.db.Attribute;
import uk.ac.ox.cs.pdq.util.Tuple;
import uk.ac.ox.cs.pdq.util.Typed;

import com.google.common.base.Preconditions;
import com.google.common.collect.Lists;

/**
 * OpenIsEmpty of the boolean query operator. Return true if the database 
 * satisfies the body of the query, false otherwise.
 * 
 * @author Julien Leblay
 */
public class OpenIsEmpty extends OpenUnaryIterator {

	/** The next tuple in the iterator. */
	private Tuple nextTuple = null;
	
	/**
	 * Instantiates a new operator.
	 * 
	 * @param child OpenTupleIterator
	 */
	public OpenIsEmpty(OpenTupleIterator child) {
		super(Lists.<Typed>newArrayList(
				new Attribute(Boolean.class, OpenIsEmpty.class.getSimpleName())),
				child);
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#deepCopy()
	 */
	@Override
	public OpenIsEmpty deepCopy() {
		return new OpenIsEmpty(this.child.deepCopy());
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.UnaryIterator#hasNext()
	 */
	@Override
	public boolean hasNext() {
		Preconditions.checkArgument(this.open);
		return this.nextTuple == null;
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#next()
	 */
	@Override
	public Tuple next() {
		Preconditions.checkArgument(this.open);
		if (this.nextTuple == null) {
			return this.getType().createTuple(!this.child.hasNext());
		}
		throw new NoSuchElementException("End of operator reached.");
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.UnaryIterator#reset()
	 */
	@Override
	public void reset() {
		super.reset();
		this.nextTuple = null;
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#getColumnsDisplay()
	 */
	@Override
	public List<String> getColumnsDisplay() {
		List<String> result = new ArrayList<>();
		result.add("IS_EMPTY");
		return result;
	}
}
