package uk.ac.ox.cs.pdq.runtime.exec.iterator.open;

import java.util.Iterator;
import java.util.List;

import uk.ac.ox.cs.pdq.util.Tuple;
import uk.ac.ox.cs.pdq.util.Typed;

import com.google.common.base.Preconditions;
import com.google.common.collect.Lists;

/**
 * OpenMemoryScan over an in-memory relation or simply a collection of tuples.
 * 
 * @author Julien Leblay
 */
public class OpenMemoryScan extends OpenTupleIterator {

	/** The next tuple to return. */
	protected Iterator<Tuple> tupleIterator = null;

	/** The next tuple to return. */
	protected Iterable<Tuple> data = null;

	/**
	 * Instantiates a new memory scan.
	 * @param columns List<Typed>
	 * @param data Iterable<Tuple>
	 */
	public OpenMemoryScan(List<Typed> columns, Iterable<Tuple> data) {
		super(Lists.<Typed>newArrayList(), columns);
		this.data = data;
	}

	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		StringBuilder result = new StringBuilder();
		result.append(this.getClass().getSimpleName());
		return result.toString();
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#open()
	 */
	@Override
	public void open() {
		this.tupleIterator = this.data.iterator();
		this.open = true;
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#close()
	 */
	@Override
	public void close() {
		// Nothing to do
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#reset()
	 */
	@Override
	public void reset() {
		this.tupleIterator = this.data.iterator();
	}


	/**
	 * @return Tuple
	 * @see java.util.Iterator#next()
	 */
	@Override
	public Tuple next() {
		Preconditions.checkState(this.open);
		return this.tupleIterator.next();
	}

	/**
	 * @return boolean
	 * @see java.util.Iterator#hasNext()
	 */
	@Override
	public boolean hasNext() {
		Preconditions.checkState(this.open);
		return !this.interrupted && this.tupleIterator.hasNext();
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#deepCopy()
	 */
	@Override
	public OpenMemoryScan deepCopy() {
		return new OpenMemoryScan(this.columns, this.data);
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#interrupt()
	 */
	@Override
	public void interrupt() {
		this.interrupted = true;
	}

	/**
	 * @param t Tuple
	 */
	@Override
	public void bind(Tuple t) {
		if (t.size() > 0) {
			throw new IllegalStateException("Bind operation not supported in Scan");
		}
	}
}