package uk.ac.ox.cs.pdq.runtime.exec.iterator.open;

import java.util.ArrayList;
import java.util.List;

import uk.ac.ox.cs.pdq.util.Tuple;
import uk.ac.ox.cs.pdq.util.TupleType;
import uk.ac.ox.cs.pdq.util.Typed;

import com.google.common.base.Preconditions;

/**
 * OpenRename physical operator.
 * 
 * @author Julien Leblay
 */
public class OpenRename extends OpenUnaryIterator {

	/**
	 * Instantiates a new rename.
	 * 
	 * @param head the head
	 * @param child OpenTupleIterator
	 */
	public OpenRename(List<Typed> head, OpenTupleIterator child) {
		super(renameInputs(head, child), child.getType(), head, child);
	}

	/**
	 * Method renameInputs.
	 * @param head List<Typed>
	 * @param child OpenTupleIterator
	 * @return List<Typed>
	 */
	private static List<Typed> renameInputs(List<Typed> head, OpenTupleIterator child) {
		TupleType newType = TupleType.DefaultFactory.createFromTyped(head);
		Preconditions.checkArgument(newType.equals(child.getType()));
		List<Typed> oldInputs= child.getInputColumns();
		List<Typed> oldOutputs = child.getColumns();
		List<Typed> result = new ArrayList<>(head.size());
		for (int i = 0, l = oldInputs.size(); i < l; i++) {
			int pos = oldOutputs.indexOf(oldInputs.get(i));
			if (pos < 0) {
				throw new IllegalStateException("Input " + oldInputs.get(i) + " not found in outputs.");
			}
			result.add(head.get(pos));
		}
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#deepCopy()
	 */
	@Override
	public OpenRename deepCopy() {
		return new OpenRename(new ArrayList<>(this.columns), this.child.deepCopy());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		StringBuilder result = new StringBuilder();
		result.append(this.getClass().getSimpleName());
		result.append('[').append(this.columns).append(']');
		result.append('(').append(this.child.toString()).append(')');
		return result.toString();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#hasNext()
	 */
	@Override
	public boolean hasNext() {
		Preconditions.checkArgument(this.open);
		return this.child.hasNext();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#next()
	 */
	@Override
	public Tuple next() {
		Preconditions.checkArgument(this.open);
		if (this.eventBus != null) {
			this.eventBus.post(this);
		}
		return this.child.next();
	}
}
