package uk.ac.ox.cs.pdq.runtime.exec.iterator.open;

import java.util.List;

import uk.ac.ox.cs.pdq.util.Tuple;
import uk.ac.ox.cs.pdq.util.TupleType;
import uk.ac.ox.cs.pdq.util.Typed;

/**
 * OpenTupleIterator defines a top-class for all open physical operators.
 * 
 * @author Julien Leblay
 */
public abstract class OpenTupleIterator
		extends uk.ac.ox.cs.pdq.runtime.exec.iterator.TupleIterator {

	/** The iterator's input type. */
	protected TupleType inputType;
	
	protected List<Typed> inputColumns;

	/**
	 * Instantiates a new operator.
	 * 
	 * @param inputColumns List<Typed>
	 * @param outputTerms List<Typed>
	 */
	public OpenTupleIterator(List<Typed> inputColumns, List<Typed> outputTerms) {
		super(outputTerms);
		this.inputColumns = inputColumns;
		this.inputType = TupleType.DefaultFactory.createFromTyped(inputColumns);
	}

	/**
	 * Instantiates a new operator.
	 * 
	 * @param inputColumns List<Typed>
	 * @param typeOverride TupleType
	 * @param outputTerms List<Typed>
	 */
	public OpenTupleIterator(List<Typed> inputColumns, TupleType typeOverride, List<Typed> outputTerms) {
		super(typeOverride, outputTerms);
		this.inputColumns = inputColumns;
		this.inputType = TupleType.DefaultFactory.createFromTyped(inputColumns);
	}

	/**
	 * Deep copy.
	 * @return a deep copy of the operator.
	 * @throws TupleIteratorException
	 * @see uk.ac.ox.cs.pdq.util.ResetableIterator#deepCopy()
	 */
	@Override
	public abstract OpenTupleIterator deepCopy();

	/**
	 * Gets the input tuple type.
	 * 
	 * @return the input type
	 */
	public TupleType getInputType() {
		return this.inputType;
	}

	/**
	 * Gets the input terms.
	 * 
	 * @return the input terms
	 */
	public List<Typed> getInputColumns() {
		return this.inputColumns;
	}
	
	/**
	 * @param t Tuple
	 */
	public abstract void bind(Tuple t);
}
