package uk.ac.ox.cs.pdq.runtime.exec.iterator.open;

import java.util.List;

import uk.ac.ox.cs.pdq.util.Tuple;
import uk.ac.ox.cs.pdq.util.TupleType;
import uk.ac.ox.cs.pdq.util.Typed;

import com.google.common.collect.Lists;
import com.google.common.eventbus.EventBus;


/**
 * Superclass to all unary logical operator.
 * 
 * @author Julien Leblay
 */
public abstract class OpenUnaryIterator extends OpenTupleIterator {

	/** The sole child of the operator */
	protected OpenTupleIterator child = null;
	
	
	/**
	 * Instantiates a new operator.
	 * 
	 * @param child OpenTupleIterator
	 */
	public OpenUnaryIterator(OpenTupleIterator child) {
		this(child.getColumns(), child);
	}
	
	/**
	 * Instantiates a new operator.
	 * 
	 * @param typeOverride TupleType
	 * @param child OpenTupleIterator
	 */
	protected OpenUnaryIterator(TupleType typeOverride, OpenTupleIterator child) {
		this(child.getInputColumns(), typeOverride, child.getColumns(), child);
	}
	
	/**
	 * Instantiates a new operator.
	 * 
	 * @param columns List<Typed>
	 * @param child OpenTupleIterator
	 */
	protected OpenUnaryIterator(List<Typed> columns, OpenTupleIterator child) {
		this(child.getInputColumns(), TupleType.DefaultFactory.createFromTyped(columns), columns, child);
	}
	
	/**
	 * Instantiates a new operator.
	 * 
	 * @param inputs List<Typed>
	 * @param typeOverride TupleType
	 * @param columns List<Typed>
	 * @param child OpenTupleIterator
	 */
	protected OpenUnaryIterator(List<Typed> inputs, TupleType typeOverride, List<Typed> columns, OpenTupleIterator child) {
		super(inputs, typeOverride, Lists.newArrayList(columns));
		this.child = child;
	}

	/**
	 * Gets the child.
	 * 
	 * @return the child
	 */
	public OpenTupleIterator getChild() {
		return this.child;
	}
	
	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#setEventBus(com.google.common.eventbus.EventBus)
	 */
	@Override
	public void setEventBus(EventBus eb) {
		super.setEventBus(eb);
		this.child.setEventBus(eb);
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#open()
	 */
	@Override
	public void open() {
		this.child.open();
		this.open = true;
	}

	/**
	 * @see java.lang.AutoCloseable#close()
	 */
	@Override
	public void close() {
		this.child.close();
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#reset()
	 */
	@Override
	public void reset() {
		this.child.reset();
	}

	/*
	 * (non-Javadoc)
	 * @see uk.ac.ox.cs.pdq.runtime.plan.physical.iterator.TupleIterator#interrupt()
	 */
	@Override
	public void interrupt() {
		this.interrupted = true;
		this.child.interrupt();
	}
	
	/**
	 * @param t Tuple
	 */
	@Override
	public void bind(Tuple t) {
		this.child.bind(t);
	}
}
