package uk.ac.ox.cs.pdq.runtime.util;

import java.io.PrintStream;

import org.apache.log4j.Logger;

import uk.ac.ox.cs.pdq.EventHandler;
import uk.ac.ox.cs.pdq.runtime.Runtime;
import uk.ac.ox.cs.pdq.util.Tuple;

import com.google.common.eventbus.Subscribe;

/**
 * Prints tuple to the given print stream, if provided, log.info otherwise.
 * 
 * @author Julien Leblay
 */
public class TuplePrinter implements EventHandler {

	/** Runner's logger. */
	private static Logger log = Logger.getLogger(Runtime.class);

	/** PrintStream where to print tuples. */
	private final PrintStream out;

	/**
	 * Default constructor
	 * @param out
	 */
	public TuplePrinter(PrintStream out) {
		this.out = out;
	}
	
	/**
	 * Empty constructor, causes the tuple to be printed to log.info.
	 */
	public TuplePrinter() {
		this(null);
	}

	/**
	 * Prints the given tuple of the default print stream, or log.info if null.
	 * @param tuple
	 */
	@Subscribe
	public void print(Tuple tuple) {
		if (this.out != null) {
			this.out.println(String.valueOf(tuple));
		} else {
			log.info(tuple);
		}
	}
}
