package uk.ac.ox.cs.pdq.test.runtime.plan.iterator;

import java.io.FileInputStream;
import java.io.IOException;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;

import uk.ac.ox.cs.pdq.algebra.predicates.AttributeEqualityPredicate;
import uk.ac.ox.cs.pdq.algebra.predicates.ConstantEqualityPredicate;
import uk.ac.ox.cs.pdq.db.Relation;
import uk.ac.ox.cs.pdq.db.Schema;
import uk.ac.ox.cs.pdq.db.TypedConstant;
import uk.ac.ox.cs.pdq.db.wrappers.RelationAccessWrapper;
import uk.ac.ox.cs.pdq.io.xml.SchemaReader;
import uk.ac.ox.cs.pdq.runtime.exec.iterator.Access;
import uk.ac.ox.cs.pdq.runtime.exec.iterator.AsymmetricMemoryHashJoin;
import uk.ac.ox.cs.pdq.runtime.exec.iterator.NestedLoopJoin;
import uk.ac.ox.cs.pdq.runtime.exec.iterator.Projection;
import uk.ac.ox.cs.pdq.runtime.exec.iterator.Scan;
import uk.ac.ox.cs.pdq.runtime.exec.iterator.Selection;
import uk.ac.ox.cs.pdq.runtime.exec.iterator.SymmetricMemoryHashJoin;
import uk.ac.ox.cs.pdq.runtime.exec.iterator.open.OpenAccess;
import uk.ac.ox.cs.pdq.runtime.exec.iterator.open.OpenBindJoin;
import uk.ac.ox.cs.pdq.runtime.exec.iterator.open.OpenCrossProduct;
import uk.ac.ox.cs.pdq.runtime.exec.iterator.open.OpenNestedLoopJoin;
import uk.ac.ox.cs.pdq.runtime.exec.iterator.open.OpenProjection;
import uk.ac.ox.cs.pdq.runtime.exec.iterator.open.OpenRename;
import uk.ac.ox.cs.pdq.runtime.exec.iterator.open.OpenScan;
import uk.ac.ox.cs.pdq.runtime.exec.iterator.open.OpenSelection;
import uk.ac.ox.cs.pdq.runtime.exec.iterator.open.OpenSymmetricMemoryHashJoin;
import uk.ac.ox.cs.pdq.util.Typed;

import com.google.common.collect.Lists;

@Ignore public class Executors {

	Schema schema = null;
	RelationAccessWrapper nation;
	RelationAccessWrapper customer;
	RelationAccessWrapper orders;

	@Before
	public void before() {
		try(FileInputStream fis = new FileInputStream("test/input/schema-fkinputs.xml")) {
			this.schema = new SchemaReader().read(fis);
			this.nation = (RelationAccessWrapper) this.schema.getRelation("nation");
			this.customer = (RelationAccessWrapper) this.schema.getRelation("customer");
			this.orders = (RelationAccessWrapper) this.schema.getRelation("orders");
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	// @Test
	// public void runCase() {
	// try ( FileInputStream pis = new
	// FileInputStream("test/input/case_009_fkinputs.xml");
	// FileInputStream qis = new
	// FileInputStream("test/input/query_case_009.xml")) {
	// Query query = Readers.with(this.schema).from(qis).read();
	// Plan plan = Readers.with(this.schema).from(pis).read();
	// BuilderParameters params = new BuilderParameters();
	// params.setExecutorType(ExecutorTypes.PIPELINED);
	// PlanExecutor executor = Middleware.newExecutor(params, plan, query);
	// } catch (IOException | EvaluationException e) {
	// e.printStackTrace();
	// }
	// }

	@Test
	public void runBottomUpNLJ1() {
		long start = System.currentTimeMillis();
		try(Scan b1 = new Scan(this.nation);
			Selection b2 = new Selection(b1, new ConstantEqualityPredicate(1, new TypedConstant<>("GERMANY")));
			Projection b3 = new Projection(Lists.newArrayList(b2.getColumn(0)), b2);
			Access b4 = new Access(this.customer, ((Relation) this.customer).getAccessMethod("m1"), b3);
			Scan b5 = new Scan(this.orders);
			NestedLoopJoin j1 = new NestedLoopJoin(new AttributeEqualityPredicate(0, 9), b4, b5);) {
			j1.open();
			int i = 0;
			while (j1.hasNext()) {
				j1.next();
				i++;
			}
			System.out.println("NLJ (customer, order) " + (System.currentTimeMillis() - start) + "\t" + i);
			Assert.assertEquals(39, i);
		}
	}

	// @Test
	// public void runBottomUpNLJ2() {
	// start = System.currentTimeMillis();
	// try ( Scan b1 = new Scan(nation);
	// Selection b2 = new Selection(b1, new ConstantEqualityPredicate(1, new
	// TypedConstant<>("GERMANY")));
	// Projection b3 = new Projection(Lists.newArrayList(b2.getColumn(0)), b2);
	// Access b4 = new Access(customer, ((Relation)
	// customer).getAccessMethodMethod("m1"), b3);
	// Scan b5 = new Scan(orders);
	// NestedLoopJoin j2 = new NestedLoopJoin(new AttributeEqualityPredicate(1,
	// 9), b5, b4);
	// ) {
	// int i = 0;
	// j2.open();
	// while (j2.hasNext()) {j2.next(); i++;}
	// System.out.println("NLJ (order, customer) " + (System.currentTimeMillis()
	// - start) + "\t" + i);
	// }
	// }
	// }

	@Test
	public void runBottomUpAHJ1() {
		long start = System.currentTimeMillis();
		try(Scan b1 = new Scan(this.nation);
			Selection b2 = new Selection(b1, new ConstantEqualityPredicate(1, new TypedConstant<>("GERMANY")));
			Projection b3 = new Projection(Lists.newArrayList(b2.getColumn(0)), b2);
			Access b4 = new Access(this.customer, ((Relation) this.customer).getAccessMethod("m1"), b3);
			Scan b5 = new Scan(this.orders);
			AsymmetricMemoryHashJoin j3 = new AsymmetricMemoryHashJoin(new AttributeEqualityPredicate(0, 9), b4, b5);) {
			int i = 0;
			j3.open();
			while (j3.hasNext()) {
				j3.next();
				i++;
			}
			System.out.println("AHJ (customer, order) " + (System.currentTimeMillis() - start) + "\t" + i);
			Assert.assertEquals(39, i);
		}
	}

	@Test
	public void runBottomUpAHJ2() {
		long start = System.currentTimeMillis();
		try(Scan b1 = new Scan(this.nation);
			Selection b2 = new Selection(b1, new ConstantEqualityPredicate(1, new TypedConstant<>("GERMANY")));
			Projection b3 = new Projection(Lists.newArrayList(b2.getColumn(0)), b2);
			Access b4 = new Access(this.customer, ((Relation) this.customer).getAccessMethod("m1"), b3);
			Scan b5 = new Scan(this.orders);
			AsymmetricMemoryHashJoin j4 = new AsymmetricMemoryHashJoin(new AttributeEqualityPredicate(1, 9), b5, b4);) {
			int i = 0;
			j4.open();
			while (j4.hasNext()) {
				j4.next();
				i++;
			}
			System.out.println("AHJ (order, customer) " + (System.currentTimeMillis() - start) + "\t" + i);
			Assert.assertEquals(39, i);
		}
	}

	@Test
	public void runBottomUpSHJ1() {
		long start = System.currentTimeMillis();
		try(Scan b1 = new Scan(this.nation);
			Selection b2 = new Selection(b1, new ConstantEqualityPredicate(1, new TypedConstant<>("GERMANY")));
			Projection b3 = new Projection(Lists.newArrayList(b2.getColumn(0)), b2);
			Access b4 = new Access(this.customer, ((Relation) this.customer).getAccessMethod("m1"), b3);
			Scan b5 = new Scan(this.orders);
			SymmetricMemoryHashJoin j5 = new SymmetricMemoryHashJoin(new AttributeEqualityPredicate(0, 9), b4, b5);) {
			int i = 0;
			j5.open();
			while (j5.hasNext()) {
				j5.next();
				i++;
			}
			System.out.println("SHJ (customer, order) " + (System.currentTimeMillis() - start) + "\t" + i);
			Assert.assertEquals(39, i);
		}
	}

	@Test
	public void runBottomUpSHJ2() {
		long start = System.currentTimeMillis();
		try(Scan b1 = new Scan(this.nation);
			Selection b2 = new Selection(b1, new ConstantEqualityPredicate(1, new TypedConstant<>("GERMANY")));
			Projection b3 = new Projection(Lists.newArrayList(b2.getColumn(0)), b2);
			Access b4 = new Access(this.customer, ((Relation) this.customer).getAccessMethod("m1"), b3);
			Scan b5 = new Scan(this.orders);
			SymmetricMemoryHashJoin j6 = new SymmetricMemoryHashJoin(new AttributeEqualityPredicate(1, 9), b5, b4);) {
			int i = 0;
			j6.open();
			while (j6.hasNext()) {
				j6.next();
				i++;
			}
			System.out.println("SHJ (order, customer) " + (System.currentTimeMillis() - start) + "\t" + i);
			Assert.assertEquals(39, i);
		}
	}

	// @Test
	// public void runTopDown() {
	// long start = System.currentTimeMillis();
	// try ( OpenScan b1 = new OpenScan(this.nation);
	// OpenSelection b2 = new OpenSelection(b1, new ConstantEqualityPredicate(1,
	// new TypedConstant<>("GERMANY")));
	// OpenProjection b3 = new
	// OpenProjection(Lists.newArrayList(b2.getColumn(0)), b2);
	// OpenRename r = new
	// OpenRename(Utility.typedToTerms(Lists.<Typed>newArrayList(this.customer.getAttributes().get(3))),
	// b3);
	// BindAccess b4 = new BindAccess(this.customer, ((Relation)
	// this.customer).getAccessMethodMethod("m1"));
	// OpenBindJoin bj = new OpenBindJoin(r, b4);
	// OpenScan b5 = new OpenScan(this.orders);
	// OpenNestedLoopJoin j1 = new OpenNestedLoopJoin(TupleType.EmptyTupleType,
	// Lists.<Term>newArrayList(), new AttributeEqualityPredicate(0, 9), bj,
	// b5);
	// ) {
	// int i = 0;
	// j1.open();
	// for (; j1.hasNext(); i++) {
	// System.out.println(j1.next());
	// }
	// System.out.println("TD (customer, order) " + (System.currentTimeMillis()
	// - start) + "\t" + i);
	// }
	// }

	@Test
	public void runTopDownNLJ1() {
		long start = System.currentTimeMillis();
		try(OpenScan b1 = new OpenScan(this.nation);
			OpenSelection b2 = new OpenSelection(b1, new ConstantEqualityPredicate(1, new TypedConstant<>("GERMANY")));
			OpenProjection b3 = new OpenProjection(Lists.newArrayList(b2.getColumn(0)), b2);
			OpenRename r = new OpenRename(Lists.<Typed> newArrayList(this.customer.getAttributes().get(3)), b3);
			OpenAccess b4 = new OpenAccess(this.customer, ((Relation) this.customer).getAccessMethod("m1"));
			OpenBindJoin bj = new OpenBindJoin(r, b4);
			OpenScan b5 = new OpenScan(this.orders);
			OpenNestedLoopJoin j1 = new OpenNestedLoopJoin(Lists.<Typed> newArrayList(),
						new AttributeEqualityPredicate(1, 10), bj, b5);) {
			j1.open();
			int i = 0;
			while (j1.hasNext()) {
				j1.next();
				i++;
			}
			System.out.println("td-NLJ (customer, order) " + (System.currentTimeMillis() - start) + "\t" + i);
			Assert.assertEquals(39, i);
		}
	}

	@Test
	public void runTopDownCP() {
		long start = System.currentTimeMillis();
		try (OpenScan b1 = new OpenScan(this.nation);
				OpenSelection b2 = new OpenSelection(b1, new ConstantEqualityPredicate(1, new TypedConstant<>("GERMANY")));
				OpenProjection b3 = new OpenProjection(Lists.newArrayList(b2.getColumn(0)), b2);
				OpenRename r = new OpenRename(Lists.<Typed> newArrayList(this.customer.getAttributes().get(3)), b3);
				OpenAccess b4 = new OpenAccess(this.customer, ((Relation) this.customer).getAccessMethod("m1"));
				OpenBindJoin bj = new OpenBindJoin(r, b4);
				OpenScan b5 = new OpenScan(this.orders);
				OpenCrossProduct j1 = new OpenCrossProduct(Lists.<Typed> newArrayList(), bj, b5);) {
			j1.open();
			int i = 0;
			while (j1.hasNext()) {
				j1.next();
				i++;
			}
			System.out.println("td-CP (customer, order) " + (System.currentTimeMillis() - start) + "\t" + i);
			Assert.assertEquals(9000, i);
		}
	}

	@Test
	public void runTopDownSHJ1() {
		long start = System.currentTimeMillis();
		try(OpenScan b1 = new OpenScan(this.nation);
			OpenSelection b2 = new OpenSelection(b1, new ConstantEqualityPredicate(1, new TypedConstant<>("GERMANY")));
			OpenProjection b3 = new OpenProjection(Lists.newArrayList(b2.getColumn(0)), b2);
			OpenRename r = new OpenRename(Lists.<Typed> newArrayList(this.customer.getAttributes().get(3)), b3);
			OpenAccess b4 = new OpenAccess(this.customer, ((Relation) this.customer).getAccessMethod("m1"));
			OpenBindJoin bj = new OpenBindJoin(r, b4);
			OpenScan b5 = new OpenScan(this.orders);
			OpenSymmetricMemoryHashJoin j1 = new OpenSymmetricMemoryHashJoin(Lists.<Typed> newArrayList(),
						new AttributeEqualityPredicate(1, 10), bj, b5);) {
			j1.open();
			int i = 0;
			while (j1.hasNext()) {
				j1.next();
				i++;
			}
			System.out.println("td-SHJ (customer, order) " + (System.currentTimeMillis() - start) + "\t" + i);
			Assert.assertEquals(39, i);
		}
	}

	@Test
	public void runTopDownSHJ2() {
		long start = System.currentTimeMillis();
		try(OpenScan b1 = new OpenScan(this.nation);
			OpenSelection b2 = new OpenSelection(b1, new ConstantEqualityPredicate(1, new TypedConstant<>("GERMANY")));
			OpenProjection b3 = new OpenProjection(Lists.newArrayList(b2.getColumn(0)), b2);
			OpenRename r = new OpenRename(Lists.<Typed> newArrayList(this.customer.getAttributes().get(3)), b3);
			OpenAccess b4 = new OpenAccess(this.customer, ((Relation) this.customer).getAccessMethod("m1"));
			OpenBindJoin bj = new OpenBindJoin(r, b4);
			OpenScan b5 = new OpenScan(this.orders);
			OpenSymmetricMemoryHashJoin j1 = new OpenSymmetricMemoryHashJoin(Lists.<Typed> newArrayList(),
						new AttributeEqualityPredicate(1, 10), b5, bj);) {
			j1.open();
			int i = 0;
			while (j1.hasNext()) {
				j1.next();
				i++;
			}
			System.out.println("td-SHJ (order, customer) " + (System.currentTimeMillis() - start) + "\t" + i);
			Assert.assertEquals(39, i);
		}
	}

	@Test
	public void runTopDownBJ1() {
		long start = System.currentTimeMillis();
		try(OpenScan b1 = new OpenScan(this.nation);
			OpenSelection b2 = new OpenSelection(b1, new ConstantEqualityPredicate(1, new TypedConstant<>("GERMANY")));
			OpenProjection b3 = new OpenProjection(Lists.newArrayList(b2.getColumn(0)), b2);
			OpenRename r1 = new OpenRename(Lists.<Typed> newArrayList(this.customer.getAttributes().get(3)), b3);
			OpenAccess b4 = new OpenAccess(this.customer, ((Relation) this.customer).getAccessMethod("m1"));
			OpenBindJoin bj = new OpenBindJoin(r1, b4);
			OpenProjection p3 = new OpenProjection(Lists.newArrayList(bj.getColumn(1)), bj);
			OpenRename r2 = new OpenRename(Lists.<Typed> newArrayList(this.orders.getAttributes().get(1)), p3);
			OpenAccess b5 = new OpenAccess(this.orders, ((Relation) this.orders).getAccessMethod("m9"));
			OpenBindJoin j1 = new OpenBindJoin(r2, b5);) {
			j1.open();
			int i = 0;
			while (j1.hasNext()) {
				j1.next();
				i++;
			}
			System.out.println("td-BJ (customer, order) " + (System.currentTimeMillis() - start) + "\t" + i);
			Assert.assertEquals(39, i);
		}
	}
}
